# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_delete_server
----------------------------------

Tests for the `delete_server` command.
"""
import uuid

from openstack import exceptions
from openstack.tests import fakes
from openstack.tests.unit import base


class TestDeleteServer(base.TestCase):
    def test_delete_server(self):
        """
        Test that server delete is called when wait=False
        """
        server = fakes.make_fake_server('1234', 'daffy', 'ACTIVE')
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', 'daffy']
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=['servers', 'detail'],
                        qs_elements=['name=daffy'],
                    ),
                    json={'servers': [server]},
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                ),
            ]
        )
        self.assertTrue(self.cloud.delete_server('daffy', wait=False))

        self.assert_calls()

    def test_delete_server_already_gone(self):
        """
        Test that we return immediately when server is already gone
        """
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', 'tweety']
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=['servers', 'detail'],
                        qs_elements=['name=tweety'],
                    ),
                    json={'servers': []},
                ),
            ]
        )
        self.assertFalse(self.cloud.delete_server('tweety', wait=False))

        self.assert_calls()

    def test_delete_server_already_gone_wait(self):
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', 'speedy']
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=['servers', 'detail'],
                        qs_elements=['name=speedy'],
                    ),
                    json={'servers': []},
                ),
            ]
        )
        self.assertFalse(self.cloud.delete_server('speedy', wait=True))
        self.assert_calls()

    def test_delete_server_wait_for_deleted(self):
        """
        Test that delete_server waits for the server to be gone
        """
        server = fakes.make_fake_server('9999', 'wily', 'ACTIVE')
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', 'wily']
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=['servers', 'detail'],
                        qs_elements=['name=wily'],
                    ),
                    json={'servers': [server]},
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '9999']
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '9999']
                    ),
                    json={'server': server},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '9999']
                    ),
                    status_code=404,
                ),
            ]
        )
        self.assertTrue(self.cloud.delete_server('wily', wait=True))

        self.assert_calls()

    def test_delete_server_fails(self):
        """
        Test that delete_server raises non-404 exceptions
        """
        server = fakes.make_fake_server('1212', 'speedy', 'ACTIVE')
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', 'speedy']
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=['servers', 'detail'],
                        qs_elements=['name=speedy'],
                    ),
                    json={'servers': [server]},
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1212']
                    ),
                    status_code=400,
                ),
            ]
        )

        self.assertRaises(
            exceptions.SDKException,
            self.cloud.delete_server,
            'speedy',
            wait=False,
        )

        self.assert_calls()

    def test_delete_server_no_cinder(self):
        """
        Test that deleting server works when cinder is not available
        """
        orig_has_service = self.cloud.has_service

        def fake_has_service(service_type):
            if service_type == 'volume':
                return False
            return orig_has_service(service_type)

        self.cloud.has_service = fake_has_service

        server = fakes.make_fake_server('1234', 'porky', 'ACTIVE')
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', 'porky']
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=['servers', 'detail'],
                        qs_elements=['name=porky'],
                    ),
                    json={'servers': [server]},
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                ),
            ]
        )
        self.assertTrue(self.cloud.delete_server('porky', wait=False))

        self.assert_calls()

    def test_delete_server_delete_ips(self):
        """
        Test that deleting server and fips works
        """
        server = fakes.make_fake_server('1234', 'porky', 'ACTIVE')
        fip_id = uuid.uuid4().hex

        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', 'porky']
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=['servers', 'detail'],
                        qs_elements=['name=porky'],
                    ),
                    json={'servers': [server]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'floatingips'],
                        qs_elements=['floating_ip_address=172.24.5.5'],
                    ),
                    complete_qs=True,
                    json={
                        'floatingips': [
                            {
                                'router_id': 'd23abc8d-2991-4a55-ba98-2aaea84cc72f',  # noqa: E501
                                'tenant_id': '4969c491a3c74ee4af974e6d800c62de',  # noqa: E501
                                'floating_network_id': '376da547-b977-4cfe-9cba7',  # noqa: E501
                                'fixed_ip_address': '10.0.0.4',
                                'floating_ip_address': '172.24.5.5',
                                'port_id': 'ce705c24-c1ef-408a-bda3-7bbd946164ac',  # noqa: E501
                                'id': fip_id,
                                'status': 'ACTIVE',
                            }
                        ]
                    },
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'floatingips', fip_id],
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'floatingips']
                    ),
                    complete_qs=True,
                    json={'floatingips': []},
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                    status_code=404,
                ),
            ]
        )
        self.assertTrue(
            self.cloud.delete_server('porky', wait=True, delete_ips=True)
        )

        self.assert_calls()

    def test_delete_server_delete_ips_bad_neutron(self):
        """
        Test that deleting server with a borked neutron doesn't bork
        """
        server = fakes.make_fake_server('1234', 'porky', 'ACTIVE')

        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', 'porky']
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=['servers', 'detail'],
                        qs_elements=['name=porky'],
                    ),
                    json={'servers': [server]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'floatingips'],
                        qs_elements=['floating_ip_address=172.24.5.5'],
                    ),
                    complete_qs=True,
                    status_code=404,
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                    status_code=404,
                ),
            ]
        )
        self.assertTrue(
            self.cloud.delete_server('porky', wait=True, delete_ips=True)
        )

        self.assert_calls()

    def test_delete_server_delete_fips_nova(self):
        """
        Test that deleting server with a borked neutron doesn't bork
        """
        self.cloud._floating_ip_source = 'nova'
        server = fakes.make_fake_server('1234', 'porky', 'ACTIVE')

        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', 'porky']
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=['servers', 'detail'],
                        qs_elements=['name=porky'],
                    ),
                    json={'servers': [server]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['os-floating-ips']
                    ),
                    json={
                        'floating_ips': [
                            {
                                'fixed_ip': None,
                                'id': 1,
                                'instance_id': None,
                                'ip': '172.24.5.5',
                                'pool': 'nova',
                            }
                        ]
                    },
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['os-floating-ips', '1']
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['os-floating-ips']
                    ),
                    json={'floating_ips': []},
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                    status_code=404,
                ),
            ]
        )
        self.assertTrue(
            self.cloud.delete_server('porky', wait=True, delete_ips=True)
        )

        self.assert_calls()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import typing as ty

from openstack.baremetal.v1 import _common
from openstack import exceptions
from openstack import resource
from openstack import utils


class Driver(resource.Resource):
    resources_key = 'drivers'
    base_path = '/drivers'

    # capabilities
    allow_create = False
    allow_fetch = True
    allow_commit = False
    allow_delete = False
    allow_list = True

    _query_mapping = resource.QueryParameters(details='detail')

    # The BIOS interface fields introduced in 1.40 (Rocky).
    # The firmware interface fields introduced in 1.86.
    _max_microversion = '1.86'

    #: A list of active hosts that support this driver.
    hosts = resource.Body('hosts', type=list)
    #: A list of relative links, including the self and bookmark links.
    links = resource.Body('links', type=list)
    #: The name of the driver
    name = resource.Body('name', alternate_id=True)
    #: A list of links to driver properties.
    properties = resource.Body('properties', type=list)

    # Hardware interface properties grouped together for convenience,
    # available with detail=True.

    #: Default BIOS interface implementation.
    #: Introduced in API microversion 1.40.
    default_bios_interface = resource.Body("default_bios_interface")
    #: Default boot interface implementation.
    #: Introduced in API microversion 1.30.
    default_boot_interface = resource.Body("default_boot_interface")
    #: Default console interface implementation.
    #: Introduced in API microversion 1.30.
    default_console_interface = resource.Body("default_console_interface")
    #: Default deploy interface implementation.
    #: Introduced in API microversion 1.30.
    default_deploy_interface = resource.Body("default_deploy_interface")
    #: Default firmware interface implementation.
    #: Introduced in API microversion 1.86.
    default_firmware_interface = resource.Body("default_firmware_interface")
    #: Default inspect interface implementation.
    #: Introduced in API microversion 1.30.
    default_inspect_interface = resource.Body("default_inspect_interface")
    #: Default management interface implementation.
    #: Introduced in API microversion 1.30.
    default_management_interface = resource.Body(
        "default_management_interface"
    )
    #: Default network interface implementation.
    #: Introduced in API microversion 1.30.
    default_network_interface = resource.Body("default_network_interface")
    #: Default port interface implementation.
    #: Introduced in API microversion 1.30.
    default_power_interface = resource.Body("default_power_interface")
    #: Default RAID interface implementation.
    #: Introduced in API microversion 1.30.
    default_raid_interface = resource.Body("default_raid_interface")
    #: Default rescue interface implementation.
    #: Introduced in API microversion 1.38.
    default_rescue_interface = resource.Body("default_rescue_interface")
    #: Default storage interface implementation.
    #: Introduced in API microversion 1.33.
    default_storage_interface = resource.Body("default_storage_interface")
    #: Default vendor interface implementation.
    #: Introduced in API microversion 1.30.
    default_vendor_interface = resource.Body("default_vendor_interface")

    #: Enabled BIOS interface implementations.
    #: Introduced in API microversion 1.40.
    enabled_bios_interfaces = resource.Body("enabled_bios_interfaces")
    #: Enabled boot interface implementations.
    #: Introduced in API microversion 1.30.
    enabled_boot_interfaces = resource.Body("enabled_boot_interfaces")
    #: Enabled console interface implementations.
    #: Introduced in API microversion 1.30.
    enabled_console_interfaces = resource.Body("enabled_console_interfaces")
    #: Enabled deploy interface implementations.
    #: Introduced in API microversion 1.30.
    enabled_deploy_interfaces = resource.Body("enabled_deploy_interfaces")
    #: Enabled firmware interface implementations.
    #: Introduced in API microversion 1.86.
    enabled_firmware_interfaces = resource.Body("enabled_firmware_interfaces")
    #: Enabled inspect interface implementations.
    #: Introduced in API microversion 1.30.
    enabled_inspect_interfaces = resource.Body("enabled_inspect_interfaces")
    #: Enabled management interface implementations.
    #: Introduced in API microversion 1.30.
    enabled_management_interfaces = resource.Body(
        "enabled_management_interfaces"
    )
    #: Enabled network interface implementations.
    #: Introduced in API microversion 1.30.
    enabled_network_interfaces = resource.Body("enabled_network_interfaces")
    #: Enabled port interface implementations.
    #: Introduced in API microversion 1.30.
    enabled_power_interfaces = resource.Body("enabled_power_interfaces")
    #: Enabled RAID interface implementations.
    #: Introduced in API microversion 1.30.
    enabled_raid_interfaces = resource.Body("enabled_raid_interfaces")
    #: Enabled rescue interface implementations.
    #: Introduced in API microversion 1.38.
    enabled_rescue_interfaces = resource.Body("enabled_rescue_interfaces")
    #: Enabled storage interface implementations.
    #: Introduced in API microversion 1.33.
    enabled_storage_interfaces = resource.Body("enabled_storage_interfaces")
    #: Enabled vendor interface implementations.
    #: Introduced in API microversion 1.30.
    enabled_vendor_interfaces = resource.Body("enabled_vendor_interfaces")

    def list_vendor_passthru(self, session):
        """Fetch vendor specific methods exposed by driver

        :param session: The session to use for making this request.
        :returns: A dict of the available vendor passthru methods for driver.
            Method names keys and corresponding usages in dict form as values
            Usage dict properties:
            * ``async``: bool # Is passthru function invoked asynchronously
            * ``attach``: bool # Is return value attached to response object
            * ``description``: str # Description of what the method does
            * ``http_methods``: list # List of HTTP methods supported
        """
        session = self._get_session(session)
        request = self._prepare_request()
        request.url = utils.urljoin(request.url, 'vendor_passthru', 'methods')
        response = session.get(request.url, headers=request.headers)

        msg = "Failed to list list vendor_passthru methods for {driver_name}"
        exceptions.raise_from_response(
            response, error_message=msg.format(driver_name=self.name)
        )
        return response.json()

    def call_vendor_passthru(
        self, session, verb: str, method: str, body: ty.Optional[dict] = None
    ):
        """Call a vendor specific passthru method

        Contents of body are params passed to the hardware driver
        function. Validation happens there. Missing parameters, or
        excess parameters will cause the request to be rejected

        :param session: The session to use for making this request.
        :param method: Vendor passthru method name.
        :param verb: One of GET, POST, PUT, DELETE,
            depending on the driver and method.
        :param body: passed to the vendor function as json body.
        :raises: :exc:`ValueError` if :data:`verb` is not one of
            GET, POST, PUT, DELETE
        :returns: response of method call.
        """
        if verb.upper() not in ['GET', 'PUT', 'POST', 'DELETE']:
            raise ValueError('Invalid verb: {}'.format(verb))

        session = self._get_session(session)
        request = self._prepare_request()
        request.url = utils.urljoin(
            request.url, f'vendor_passthru?method={method}'
        )
        call = getattr(session, verb.lower())
        response = call(
            request.url,
            json=body,
            headers=request.headers,
            retriable_status_codes=_common.RETRIABLE_STATUS_CODES,
        )

        msg = "Failed call to method {method} on driver {driver_name}".format(
            method=method, driver_name=self.name
        )
        exceptions.raise_from_response(response, error_message=msg)
        return response



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.image.v2 import image as _image
from openstack.tests.functional.image.v2 import base

# NOTE(stephenfin): This is referenced in the Compute functional tests to avoid
# attempts to boot from it.
TEST_IMAGE_NAME = 'Test Image'


class TestImage(base.BaseImageTest):
    def setUp(self):
        super().setUp()

        # there's a limit on name length
        self.image = self.conn.image.create_image(
            name=TEST_IMAGE_NAME,
            disk_format='raw',
            container_format='bare',
            properties={
                'description': 'This is not an image',
            },
            data=open('CONTRIBUTING.rst', 'r'),
        )
        self.assertIsInstance(self.image, _image.Image)
        self.assertEqual(TEST_IMAGE_NAME, self.image.name)

    def tearDown(self):
        # we do this in tearDown rather than via 'addCleanup' since we want to
        # wait for the deletion of the resource to ensure it completes
        self.conn.image.delete_image(self.image)
        self.conn.image.wait_for_delete(self.image)

        super().tearDown()

    def test_images(self):
        # get image
        image = self.conn.image.get_image(self.image.id)
        self.assertEqual(self.image.name, image.name)

        # find image
        image = self.conn.image.find_image(self.image.name)
        self.assertEqual(self.image.id, image.id)

        # list
        images = list(self.conn.image.images())
        # there are many other images so we don't assert that this is the
        # *only* image present
        self.assertIn(self.image.id, {i.id for i in images})

        # update
        image_name = self.getUniqueString()
        image = self.conn.image.update_image(
            self.image,
            name=image_name,
        )
        self.assertIsInstance(image, _image.Image)
        image = self.conn.image.get_image(self.image.id)
        self.assertEqual(image_name, image.name)

    def test_tags(self):
        # add tag
        image = self.conn.image.get_image(self.image)
        self.conn.image.add_tag(image, 't1')
        self.conn.image.add_tag(image, 't2')

        # filter image by tags
        image = list(self.conn.image.images(tag=['t1', 't2']))[0]
        self.assertEqual(image.id, image.id)
        self.assertIn('t1', image.tags)
        self.assertIn('t2', image.tags)

        # remove tag
        self.conn.image.remove_tag(image, 't1')
        image = self.conn.image.get_image(self.image)
        self.assertIn('t2', image.tags)
        self.assertNotIn('t1', image.tags)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
from unittest import mock

from keystoneauth1 import adapter

from openstack.compute.v2 import limits
from openstack.tests.unit import base

ABSOLUTE_LIMITS = {
    "maxImageMeta": 128,
    "maxSecurityGroupRules": 20,
    "maxSecurityGroups": 10,
    "maxServerMeta": 128,
    "maxTotalCores": 20,
    "maxTotalFloatingIps": 10,
    "maxTotalInstances": 10,
    "maxTotalKeypairs": 100,
    "maxTotalRAMSize": 51200,
    "maxServerGroups": 10,
    "maxServerGroupMembers": 10,
    "totalFloatingIpsUsed": 1,
    "totalSecurityGroupsUsed": 2,
    "totalRAMUsed": 4,
    "totalInstancesUsed": 5,
    "totalServerGroupsUsed": 6,
    "totalCoresUsed": 7,
}

RATE_LIMIT = {
    "limit": [
        {
            "next-available": "2012-11-27T17:22:18Z",
            "remaining": 120,
            "unit": "MINUTE",
            "value": 120,
            "verb": "POST",
        },
    ],
    "regex": ".*",
    "uri": "*",
}

LIMITS_BODY = {"limits": {"absolute": ABSOLUTE_LIMITS, "rate": [RATE_LIMIT]}}


class TestAbsoluteLimits(base.TestCase):
    def test_basic(self):
        sot = limits.AbsoluteLimits()
        self.assertIsNone(sot.resource_key)
        self.assertIsNone(sot.resources_key)
        self.assertEqual("", sot.base_path)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_list)

    def test_make_it(self):
        sot = limits.AbsoluteLimits(**ABSOLUTE_LIMITS)
        self.assertEqual(ABSOLUTE_LIMITS["maxImageMeta"], sot.image_meta)
        self.assertEqual(
            ABSOLUTE_LIMITS["maxSecurityGroupRules"], sot.security_group_rules
        )
        self.assertEqual(
            ABSOLUTE_LIMITS["maxSecurityGroups"], sot.security_groups
        )
        self.assertEqual(ABSOLUTE_LIMITS["maxServerMeta"], sot.server_meta)
        self.assertEqual(ABSOLUTE_LIMITS["maxTotalCores"], sot.total_cores)
        self.assertEqual(
            ABSOLUTE_LIMITS["maxTotalFloatingIps"], sot.floating_ips
        )
        self.assertEqual(ABSOLUTE_LIMITS["maxTotalInstances"], sot.instances)
        self.assertEqual(ABSOLUTE_LIMITS["maxTotalKeypairs"], sot.keypairs)
        self.assertEqual(ABSOLUTE_LIMITS["maxTotalRAMSize"], sot.total_ram)
        self.assertEqual(ABSOLUTE_LIMITS["maxServerGroups"], sot.server_groups)
        self.assertEqual(
            ABSOLUTE_LIMITS["maxServerGroupMembers"], sot.server_group_members
        )
        self.assertEqual(
            ABSOLUTE_LIMITS["totalFloatingIpsUsed"], sot.floating_ips_used
        )
        self.assertEqual(
            ABSOLUTE_LIMITS["totalSecurityGroupsUsed"],
            sot.security_groups_used,
        )
        self.assertEqual(ABSOLUTE_LIMITS["totalRAMUsed"], sot.total_ram_used)
        self.assertEqual(
            ABSOLUTE_LIMITS["totalInstancesUsed"], sot.instances_used
        )
        self.assertEqual(
            ABSOLUTE_LIMITS["totalServerGroupsUsed"], sot.server_groups_used
        )
        self.assertEqual(
            ABSOLUTE_LIMITS["totalCoresUsed"], sot.total_cores_used
        )


class TestRateLimit(base.TestCase):
    def test_basic(self):
        sot = limits.RateLimit()
        self.assertIsNone(sot.resource_key)
        self.assertIsNone(sot.resources_key)
        self.assertEqual("", sot.base_path)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_list)

    def test_make_it(self):
        sot = limits.RateLimit(**RATE_LIMIT)
        self.assertEqual(RATE_LIMIT["regex"], sot.regex)
        self.assertEqual(RATE_LIMIT["uri"], sot.uri)
        self.assertEqual(RATE_LIMIT["limit"], sot.limits)


class TestLimits(base.TestCase):
    def test_basic(self):
        sot = limits.Limits()
        self.assertEqual("limits", sot.resource_key)
        self.assertEqual("/limits", sot.base_path)
        self.assertTrue(sot.allow_fetch)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_list)
        self.assertDictEqual(
            {'limit': 'limit', 'marker': 'marker', 'tenant_id': 'tenant_id'},
            sot._query_mapping._mapping,
        )

    def test_get(self):
        sess = mock.Mock(spec=adapter.Adapter)
        sess.default_microversion = None
        resp = mock.Mock()
        sess.get.return_value = resp
        resp.json.return_value = copy.deepcopy(LIMITS_BODY)
        resp.headers = {}
        resp.status_code = 200

        sot = limits.Limits().fetch(sess)

        self.assertEqual(
            ABSOLUTE_LIMITS["maxImageMeta"], sot.absolute.image_meta
        )
        self.assertEqual(
            ABSOLUTE_LIMITS["maxSecurityGroupRules"],
            sot.absolute.security_group_rules,
        )
        self.assertEqual(
            ABSOLUTE_LIMITS["maxSecurityGroups"], sot.absolute.security_groups
        )
        self.assertEqual(
            ABSOLUTE_LIMITS["maxServerMeta"], sot.absolute.server_meta
        )
        self.assertEqual(
            ABSOLUTE_LIMITS["maxTotalCores"], sot.absolute.total_cores
        )
        self.assertEqual(
            ABSOLUTE_LIMITS["maxTotalFloatingIps"], sot.absolute.floating_ips
        )
        self.assertEqual(
            ABSOLUTE_LIMITS["maxTotalInstances"], sot.absolute.instances
        )
        self.assertEqual(
            ABSOLUTE_LIMITS["maxTotalKeypairs"], sot.absolute.keypairs
        )
        self.assertEqual(
            ABSOLUTE_LIMITS["maxTotalRAMSize"], sot.absolute.total_ram
        )
        self.assertEqual(
            ABSOLUTE_LIMITS["maxServerGroups"], sot.absolute.server_groups
        )
        self.assertEqual(
            ABSOLUTE_LIMITS["maxServerGroupMembers"],
            sot.absolute.server_group_members,
        )
        self.assertEqual(
            ABSOLUTE_LIMITS["totalFloatingIpsUsed"],
            sot.absolute.floating_ips_used,
        )
        self.assertEqual(
            ABSOLUTE_LIMITS["totalSecurityGroupsUsed"],
            sot.absolute.security_groups_used,
        )
        self.assertEqual(
            ABSOLUTE_LIMITS["totalRAMUsed"], sot.absolute.total_ram_used
        )
        self.assertEqual(
            ABSOLUTE_LIMITS["totalInstancesUsed"], sot.absolute.instances_used
        )
        self.assertEqual(
            ABSOLUTE_LIMITS["totalServerGroupsUsed"],
            sot.absolute.server_groups_used,
        )
        self.assertEqual(
            ABSOLUTE_LIMITS["totalCoresUsed"], sot.absolute.total_cores_used
        )

        self.assertEqual(RATE_LIMIT["uri"], sot.rate[0].uri)
        self.assertEqual(RATE_LIMIT["regex"], sot.rate[0].regex)
        self.assertEqual(RATE_LIMIT["limit"], sot.rate[0].limits)

        dsot = sot.to_dict()

        self.assertIsInstance(dsot['rate'][0], dict)
        self.assertIsInstance(dsot['absolute'], dict)
        self.assertEqual(RATE_LIMIT["uri"], dsot['rate'][0]['uri'])
        self.assertEqual(
            ABSOLUTE_LIMITS["totalSecurityGroupsUsed"],
            dsot['absolute']['security_groups_used'],
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.dns.v2 import recordset
from openstack.tests.unit import base


IDENTIFIER = 'NAME'
EXAMPLE = {
    'description': 'This is an example record set.',
    'updated_at': None,
    'records': ['10.1.0.2'],
    'ttl': 3600,
    'id': IDENTIFIER,
    'name': 'example.org.',
    'project_id': '4335d1f0-f793-11e2-b778-0800200c9a66',
    'zone_id': '2150b1bf-dee2-4221-9d85-11f7886fb15f',
    'zone_name': 'example.com.',
    'created_at': '2014-10-24T19:59:44.000000',
    'version': 1,
    'type': 'A',
    'status': 'ACTIVE',
    'action': 'NONE',
}


class TestRecordset(base.TestCase):
    def test_basic(self):
        sot = recordset.Recordset()
        self.assertIsNone(sot.resource_key)
        self.assertEqual('recordsets', sot.resources_key)
        self.assertEqual('/zones/%(zone_id)s/recordsets', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

        self.assertDictEqual(
            {
                'data': 'data',
                'description': 'description',
                'limit': 'limit',
                'marker': 'marker',
                'name': 'name',
                'status': 'status',
                'ttl': 'ttl',
                'type': 'type',
            },
            sot._query_mapping._mapping,
        )

    def test_make_it(self):
        sot = recordset.Recordset(**EXAMPLE)
        self.assertEqual(IDENTIFIER, sot.id)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['ttl'], sot.ttl)
        self.assertEqual(EXAMPLE['type'], sot.type)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['status'], sot.status)



# Copyright 2018 Red Hat, Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from testscenarios import load_tests_apply_scenarios as load_tests  # noqa

from openstack.config import cloud_region
from openstack import connection
from openstack.tests import fakes
from openstack.tests.unit import base


class TestFromSession(base.TestCase):
    scenarios = [
        ('no_region', dict(test_region=None)),
        ('with_region', dict(test_region='RegionOne')),
    ]

    def test_from_session(self):
        config = cloud_region.from_session(
            self.cloud.session, region_name=self.test_region
        )
        self.assertEqual(config.name, 'identity.example.com')
        if not self.test_region:
            self.assertIsNone(config.region_name)
        else:
            self.assertEqual(config.region_name, self.test_region)

        server_id = str(uuid.uuid4())
        server_name = self.getUniqueString('name')
        fake_server = fakes.make_fake_server(server_id, server_name)
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', 'detail']
                    ),
                    json={'servers': [fake_server]},
                ),
            ]
        )

        conn = connection.Connection(config=config)
        s = next(conn.compute.servers())
        self.assertEqual(s.id, server_id)
        self.assertEqual(s.name, server_name)
        self.assert_calls()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_create_server
----------------------------------

Tests for the `create_server` command.
"""

import base64
from unittest import mock
import uuid

from openstack.cloud import meta
from openstack.compute.v2 import server
from openstack import connection
from openstack import exceptions
from openstack.tests import fakes
from openstack.tests.unit import base


class TestCreateServer(base.TestCase):
    def _compare_servers(self, exp, real):
        self.assertDictEqual(
            server.Server(**exp).to_dict(computed=False),
            real.to_dict(computed=False),
        )

    def test_create_server_with_get_exception(self):
        """
        Test that a bad status code when attempting to get the server instance
        raises an exception in create_server.
        """
        build_server = fakes.make_fake_server('1234', '', 'BUILD')
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json={'networks': []},
                ),
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers']
                    ),
                    json={'server': build_server},
                    validate=dict(
                        json={
                            'server': {
                                'flavorRef': 'flavor-id',
                                'imageRef': 'image-id',
                                'max_count': 1,
                                'min_count': 1,
                                'name': 'server-name',
                                'networks': 'auto',
                            }
                        }
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                    status_code=404,
                ),
            ]
        )
        self.assertRaises(
            exceptions.SDKException,
            self.cloud.create_server,
            'server-name',
            {'id': 'image-id'},
            {'id': 'flavor-id'},
        )
        self.assert_calls()

    def test_create_server_with_server_error(self):
        """
        Test that a server error before we return or begin waiting for the
        server instance spawn raises an exception in create_server.
        """
        build_server = fakes.make_fake_server('1234', '', 'BUILD')
        error_server = fakes.make_fake_server('1234', '', 'ERROR')
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json={'networks': []},
                ),
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers']
                    ),
                    json={'server': build_server},
                    validate=dict(
                        json={
                            'server': {
                                'flavorRef': 'flavor-id',
                                'imageRef': 'image-id',
                                'max_count': 1,
                                'min_count': 1,
                                'name': 'server-name',
                                'networks': 'auto',
                            }
                        }
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                    json={'server': error_server},
                ),
            ]
        )
        self.assertRaises(
            exceptions.SDKException,
            self.cloud.create_server,
            'server-name',
            {'id': 'image-id'},
            {'id': 'flavor-id'},
        )
        self.assert_calls()

    def test_create_server_wait_server_error(self):
        """
        Test that a server error while waiting for the server to spawn
        raises an exception in create_server.
        """
        build_server = fakes.make_fake_server('1234', '', 'BUILD')
        error_server = fakes.make_fake_server('1234', '', 'ERROR')
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json={'networks': []},
                ),
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers']
                    ),
                    json={'server': build_server},
                    validate=dict(
                        json={
                            'server': {
                                'flavorRef': 'flavor-id',
                                'imageRef': 'image-id',
                                'max_count': 1,
                                'min_count': 1,
                                'name': 'server-name',
                                'networks': 'auto',
                            }
                        }
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', 'detail']
                    ),
                    json={'servers': [build_server]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', 'detail']
                    ),
                    json={'servers': [error_server]},
                ),
            ]
        )
        self.assertRaises(
            exceptions.SDKException,
            self.cloud.create_server,
            'server-name',
            dict(id='image-id'),
            dict(id='flavor-id'),
            wait=True,
        )

        self.assert_calls()

    def test_create_server_with_timeout(self):
        """
        Test that a timeout while waiting for the server to spawn raises an
        exception in create_server.
        """
        fake_server = fakes.make_fake_server('1234', '', 'BUILD')
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json={'networks': []},
                ),
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers']
                    ),
                    json={'server': fake_server},
                    validate=dict(
                        json={
                            'server': {
                                'flavorRef': 'flavor-id',
                                'imageRef': 'image-id',
                                'max_count': 1,
                                'min_count': 1,
                                'name': 'server-name',
                                'networks': 'auto',
                            }
                        }
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', 'detail']
                    ),
                    json={'servers': [fake_server]},
                ),
            ]
        )
        self.assertRaises(
            exceptions.ResourceTimeout,
            self.cloud.create_server,
            'server-name',
            dict(id='image-id'),
            dict(id='flavor-id'),
            wait=True,
            timeout=0.01,
        )
        # We poll at the end, so we don't know real counts
        self.assert_calls(do_count=False)

    def test_create_server_no_wait(self):
        """
        Test that create_server with no wait and no exception in the
        create call returns the server instance.
        """
        fake_server = fakes.make_fake_server('1234', '', 'BUILD')
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json={'networks': []},
                ),
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers']
                    ),
                    json={'server': fake_server},
                    validate=dict(
                        json={
                            'server': {
                                'flavorRef': 'flavor-id',
                                'imageRef': 'image-id',
                                'max_count': 1,
                                'min_count': 1,
                                'name': 'server-name',
                                'networks': 'auto',
                            }
                        }
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                    json={'server': fake_server},
                ),
            ]
        )
        self.assertDictEqual(
            server.Server(**fake_server).to_dict(computed=False),
            self.cloud.create_server(
                name='server-name',
                image=dict(id='image-id'),
                flavor=dict(id='flavor-id'),
            ).to_dict(computed=False),
        )

        self.assert_calls()

    def test_create_server_config_drive(self):
        """
        Test that config_drive gets passed in properly
        """
        fake_server = fakes.make_fake_server('1234', '', 'BUILD')
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json={'networks': []},
                ),
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers']
                    ),
                    json={'server': fake_server},
                    validate=dict(
                        json={
                            'server': {
                                'flavorRef': 'flavor-id',
                                'imageRef': 'image-id',
                                'config_drive': True,
                                'max_count': 1,
                                'min_count': 1,
                                'name': 'server-name',
                                'networks': 'auto',
                            }
                        }
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                    json={'server': fake_server},
                ),
            ]
        )
        self.assertDictEqual(
            server.Server(**fake_server).to_dict(computed=False),
            self.cloud.create_server(
                name='server-name',
                image=dict(id='image-id'),
                flavor=dict(id='flavor-id'),
                config_drive=True,
            ).to_dict(computed=False),
        )

        self.assert_calls()

    def test_create_server_config_drive_none(self):
        """
        Test that config_drive gets not passed in properly
        """
        fake_server = fakes.make_fake_server('1234', '', 'BUILD')
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json={'networks': []},
                ),
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers']
                    ),
                    json={'server': fake_server},
                    validate=dict(
                        json={
                            'server': {
                                'flavorRef': 'flavor-id',
                                'imageRef': 'image-id',
                                'max_count': 1,
                                'min_count': 1,
                                'name': 'server-name',
                                'networks': 'auto',
                            }
                        }
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                    json={'server': fake_server},
                ),
            ]
        )
        self.assertEqual(
            server.Server(**fake_server).to_dict(computed=False),
            self.cloud.create_server(
                name='server-name',
                image=dict(id='image-id'),
                flavor=dict(id='flavor-id'),
                config_drive=None,
            ).to_dict(computed=False),
        )

        self.assert_calls()

    def test_create_server_with_admin_pass_no_wait(self):
        """
        Test that a server with an admin_pass passed returns the password
        """
        admin_pass = self.getUniqueString('password')
        fake_server = fakes.make_fake_server('1234', '', 'BUILD')
        fake_create_server = fakes.make_fake_server(
            '1234', '', 'BUILD', admin_pass=admin_pass
        )
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json={'networks': []},
                ),
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers']
                    ),
                    json={'server': fake_create_server},
                    validate=dict(
                        json={
                            'server': {
                                'adminPass': admin_pass,
                                'flavorRef': 'flavor-id',
                                'imageRef': 'image-id',
                                'max_count': 1,
                                'min_count': 1,
                                'name': 'server-name',
                                'networks': 'auto',
                            }
                        }
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                    json={'server': fake_server},
                ),
            ]
        )
        self.assertEqual(
            admin_pass,
            self.cloud.create_server(
                name='server-name',
                image=dict(id='image-id'),
                flavor=dict(id='flavor-id'),
                admin_pass=admin_pass,
            )['admin_password'],
        )

        self.assert_calls()

    @mock.patch.object(connection.Connection, "wait_for_server")
    def test_create_server_with_admin_pass_wait(self, mock_wait):
        """
        Test that a server with an admin_pass passed returns the password
        """
        admin_pass = self.getUniqueString('password')
        fake_server = fakes.make_fake_server('1234', '', 'BUILD')
        fake_server_with_pass = fakes.make_fake_server(
            '1234', '', 'BUILD', admin_pass=admin_pass
        )
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json={'networks': []},
                ),
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers']
                    ),
                    json={'server': fake_server_with_pass},
                    validate=dict(
                        json={
                            'server': {
                                'flavorRef': 'flavor-id',
                                'imageRef': 'image-id',
                                'max_count': 1,
                                'min_count': 1,
                                'adminPass': admin_pass,
                                'name': 'server-name',
                                'networks': 'auto',
                            }
                        }
                    ),
                ),
            ]
        )

        # The wait returns non-password server
        mock_wait.return_value = server.Server(**fake_server)

        new_server = self.cloud.create_server(
            name='server-name',
            image=dict(id='image-id'),
            flavor=dict(id='flavor-id'),
            admin_pass=admin_pass,
            wait=True,
        )

        # Assert that we did wait
        self.assertTrue(mock_wait.called)

        # Even with the wait, we should still get back a passworded server
        self.assertEqual(
            new_server['admin_password'], fake_server_with_pass['adminPass']
        )
        self.assert_calls()

    def test_create_server_user_data_base64(self):
        """
        Test that a server passed user-data sends it base64 encoded.
        """
        user_data = self.getUniqueString('user_data')
        user_data_b64 = base64.b64encode(user_data.encode('utf-8')).decode(
            'utf-8'
        )
        fake_server = fakes.make_fake_server('1234', '', 'BUILD')
        fake_server['user_data'] = user_data

        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json={'networks': []},
                ),
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers']
                    ),
                    json={'server': fake_server},
                    validate=dict(
                        json={
                            'server': {
                                'flavorRef': 'flavor-id',
                                'imageRef': 'image-id',
                                'max_count': 1,
                                'min_count': 1,
                                'user_data': user_data_b64,
                                'name': 'server-name',
                                'networks': 'auto',
                            }
                        }
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                    json={'server': fake_server},
                ),
            ]
        )

        self.cloud.create_server(
            name='server-name',
            image=dict(id='image-id'),
            flavor=dict(id='flavor-id'),
            userdata=user_data,
            wait=False,
        )

        self.assert_calls()

    @mock.patch.object(connection.Connection, "get_active_server")
    @mock.patch.object(connection.Connection, "get_server")
    def test_wait_for_server(self, mock_get_server, mock_get_active_server):
        """
        Test that waiting for a server returns the server instance when
        its status changes to "ACTIVE".
        """
        # TODO(mordred) Rework this to not mock methods
        building_server = {'id': 'fake_server_id', 'status': 'BUILDING'}
        active_server = {'id': 'fake_server_id', 'status': 'ACTIVE'}

        mock_get_server.side_effect = iter([building_server, active_server])
        mock_get_active_server.side_effect = iter(
            [building_server, active_server]
        )

        server = self.cloud.wait_for_server(building_server)

        self.assertEqual(2, mock_get_server.call_count)
        mock_get_server.assert_has_calls(
            [
                mock.call(building_server['id']),
                mock.call(active_server['id']),
            ]
        )

        self.assertEqual(2, mock_get_active_server.call_count)
        mock_get_active_server.assert_has_calls(
            [
                mock.call(
                    server=building_server,
                    reuse=True,
                    auto_ip=True,
                    ips=None,
                    ip_pool=None,
                    wait=True,
                    timeout=mock.ANY,
                    nat_destination=None,
                ),
                mock.call(
                    server=active_server,
                    reuse=True,
                    auto_ip=True,
                    ips=None,
                    ip_pool=None,
                    wait=True,
                    timeout=mock.ANY,
                    nat_destination=None,
                ),
            ]
        )

        self.assertEqual('ACTIVE', server['status'])

    @mock.patch.object(connection.Connection, 'wait_for_server')
    def test_create_server_wait(self, mock_wait):
        """
        Test that create_server with a wait actually does the wait.
        """
        # TODO(mordred) Make this a full proper response
        fake_server = fakes.make_fake_server('1234', '', 'BUILD')

        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json={'networks': []},
                ),
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers']
                    ),
                    json={'server': fake_server},
                    validate=dict(
                        json={
                            'server': {
                                'flavorRef': 'flavor-id',
                                'imageRef': 'image-id',
                                'max_count': 1,
                                'min_count': 1,
                                'name': 'server-name',
                                'networks': 'auto',
                            }
                        }
                    ),
                ),
            ]
        )
        self.cloud.create_server(
            'server-name',
            dict(id='image-id'),
            dict(id='flavor-id'),
            wait=True,
        ),

        # This is a pretty dirty hack to ensure we in principle use object with
        # expected properties
        srv = server.Server.existing(
            connection=self.cloud,
            min_count=1,
            max_count=1,
            networks='auto',
            imageRef='image-id',
            flavorRef='flavor-id',
            **fake_server
        )
        mock_wait.assert_called_once_with(
            srv,
            auto_ip=True,
            ips=None,
            ip_pool=None,
            reuse=True,
            timeout=180,
            nat_destination=None,
        )
        self.assert_calls()

    @mock.patch.object(connection.Connection, 'add_ips_to_server')
    def test_create_server_no_addresses(self, mock_add_ips_to_server):
        """
        Test that create_server with a wait throws an exception if the
        server doesn't have addresses.
        """
        build_server = fakes.make_fake_server('1234', '', 'BUILD')
        fake_server = fakes.make_fake_server(
            '1234', '', 'ACTIVE', addresses={}
        )
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json={'networks': []},
                ),
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers']
                    ),
                    json={'server': build_server},
                    validate=dict(
                        json={
                            'server': {
                                'flavorRef': 'flavor-id',
                                'imageRef': 'image-id',
                                'max_count': 1,
                                'min_count': 1,
                                'name': 'server-name',
                                'networks': 'auto',
                            }
                        }
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', 'detail']
                    ),
                    json={'servers': [build_server]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', 'detail']
                    ),
                    json={'servers': [fake_server]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'ports'],
                        qs_elements=['device_id=1234'],
                    ),
                    json={'ports': []},
                ),
                dict(
                    method='DELETE',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                    status_code=404,
                ),
            ]
        )
        mock_add_ips_to_server.return_value = fake_server

        self.assertRaises(
            exceptions.SDKException,
            self.cloud.create_server,
            'server-name',
            {'id': 'image-id'},
            {'id': 'flavor-id'},
            wait=True,
        )

        self.assert_calls()

    def test_create_server_network_with_no_nics(self):
        """
        Verify that if 'network' is supplied, and 'nics' is not, that we
        attempt to get the network for the server.
        """
        build_server = fakes.make_fake_server('1234', '', 'BUILD')
        network = {'id': 'network-id', 'name': 'network-name'}
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'networks', 'network-name'],
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'networks'],
                        qs_elements=['name=network-name'],
                    ),
                    json={'networks': [network]},
                ),
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers']
                    ),
                    json={'server': build_server},
                    validate=dict(
                        json={
                            'server': {
                                'flavorRef': 'flavor-id',
                                'imageRef': 'image-id',
                                'max_count': 1,
                                'min_count': 1,
                                'networks': [{'uuid': 'network-id'}],
                                'name': 'server-name',
                            }
                        }
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                    json={'server': build_server},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json={'networks': [network]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'subnets']
                    ),
                    json={'subnets': []},
                ),
            ]
        )
        self.cloud.create_server(
            'server-name',
            dict(id='image-id'),
            dict(id='flavor-id'),
            network='network-name',
        )
        self.assert_calls()

    def test_create_server_network_with_empty_nics(self):
        """
        Verify that if 'network' is supplied, along with an empty 'nics' list,
        it's treated the same as if 'nics' were not included.
        """
        network = {'id': 'network-id', 'name': 'network-name'}
        build_server = fakes.make_fake_server('1234', '', 'BUILD')
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'networks', 'network-name'],
                    ),
                    status_code=404,
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network',
                        'public',
                        append=['v2.0', 'networks'],
                        qs_elements=['name=network-name'],
                    ),
                    json={'networks': [network]},
                ),
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers']
                    ),
                    json={'server': build_server},
                    validate=dict(
                        json={
                            'server': {
                                'flavorRef': 'flavor-id',
                                'imageRef': 'image-id',
                                'max_count': 1,
                                'min_count': 1,
                                'networks': [{'uuid': 'network-id'}],
                                'name': 'server-name',
                            }
                        }
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                    json={'server': build_server},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json={'networks': [network]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'subnets']
                    ),
                    json={'subnets': []},
                ),
            ]
        )
        self.cloud.create_server(
            'server-name',
            dict(id='image-id'),
            dict(id='flavor-id'),
            network='network-name',
            nics=[],
        )
        self.assert_calls()

    def test_create_server_network_fixed_ip(self):
        """
        Verify that if 'fixed_ip' is supplied in nics, we pass it to networks
        appropriately.
        """
        network = {'id': 'network-id', 'name': 'network-name'}
        fixed_ip = '10.0.0.1'
        build_server = fakes.make_fake_server('1234', '', 'BUILD')
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers']
                    ),
                    json={'server': build_server},
                    validate=dict(
                        json={
                            'server': {
                                'flavorRef': 'flavor-id',
                                'imageRef': 'image-id',
                                'max_count': 1,
                                'min_count': 1,
                                'networks': [{'fixed_ip': fixed_ip}],
                                'name': 'server-name',
                            }
                        }
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                    json={'server': build_server},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json={'networks': [network]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'subnets']
                    ),
                    json={'subnets': []},
                ),
            ]
        )
        self.cloud.create_server(
            'server-name',
            dict(id='image-id'),
            dict(id='flavor-id'),
            nics=[{'fixed_ip': fixed_ip}],
        )
        self.assert_calls()

    def test_create_server_network_v4_fixed_ip(self):
        """
        Verify that if 'v4-fixed-ip' is supplied in nics, we pass it to
        networks appropriately.
        """
        network = {'id': 'network-id', 'name': 'network-name'}
        fixed_ip = '10.0.0.1'
        build_server = fakes.make_fake_server('1234', '', 'BUILD')
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers']
                    ),
                    json={'server': build_server},
                    validate=dict(
                        json={
                            'server': {
                                'flavorRef': 'flavor-id',
                                'imageRef': 'image-id',
                                'max_count': 1,
                                'min_count': 1,
                                'networks': [{'fixed_ip': fixed_ip}],
                                'name': 'server-name',
                            }
                        }
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                    json={'server': build_server},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json={'networks': [network]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'subnets']
                    ),
                    json={'subnets': []},
                ),
            ]
        )
        self.cloud.create_server(
            'server-name',
            dict(id='image-id'),
            dict(id='flavor-id'),
            nics=[{'fixed_ip': fixed_ip}],
        )
        self.assert_calls()

    def test_create_server_network_v6_fixed_ip(self):
        """
        Verify that if 'v6-fixed-ip' is supplied in nics, we pass it to
        networks appropriately.
        """
        network = {'id': 'network-id', 'name': 'network-name'}
        # Note - it doesn't actually have to be a v6 address - it's just
        # an alias.
        fixed_ip = 'fe80::28da:5fff:fe57:13ed'
        build_server = fakes.make_fake_server('1234', '', 'BUILD')
        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers']
                    ),
                    json={'server': build_server},
                    validate=dict(
                        json={
                            'server': {
                                'flavorRef': 'flavor-id',
                                'imageRef': 'image-id',
                                'max_count': 1,
                                'min_count': 1,
                                'networks': [{'fixed_ip': fixed_ip}],
                                'name': 'server-name',
                            }
                        }
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                    json={'server': build_server},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json={'networks': [network]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'subnets']
                    ),
                    json={'subnets': []},
                ),
            ]
        )
        self.cloud.create_server(
            'server-name',
            dict(id='image-id'),
            dict(id='flavor-id'),
            nics=[{'fixed_ip': fixed_ip}],
        )
        self.assert_calls()

    def test_create_server_network_fixed_ip_conflicts(self):
        """
        Verify that if 'fixed_ip' and 'v4-fixed-ip' are both supplied in nics,
        we throw an exception.
        """
        # Note - it doesn't actually have to be a v6 address - it's just
        # an alias.
        self.use_nothing()
        fixed_ip = '10.0.0.1'
        self.assertRaises(
            exceptions.SDKException,
            self.cloud.create_server,
            'server-name',
            dict(id='image-id'),
            dict(id='flavor-id'),
            nics=[{'fixed_ip': fixed_ip, 'v4-fixed-ip': fixed_ip}],
        )
        self.assert_calls()

    def test_create_server_get_flavor_image(self):
        self.use_glance()
        image_id = str(uuid.uuid4())
        fake_image_dict = fakes.make_fake_image(image_id=image_id)
        fake_image_search_return = {'images': [fake_image_dict]}

        build_server = fakes.make_fake_server('1234', '', 'BUILD')
        active_server = fakes.make_fake_server('1234', '', 'BUILD')

        self.register_uris(
            [
                dict(
                    method='GET',
                    uri='https://image.example.com/v2/images',
                    json=fake_image_search_return,
                ),
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute',
                        'public',
                        append=['flavors', 'vanilla'],
                        qs_elements=[],
                    ),
                    json=fakes.FAKE_FLAVOR,
                ),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers']
                    ),
                    json={'server': build_server},
                    validate=dict(
                        json={
                            'server': {
                                'flavorRef': fakes.FLAVOR_ID,
                                'imageRef': image_id,
                                'max_count': 1,
                                'min_count': 1,
                                'networks': [{'uuid': 'some-network'}],
                                'name': 'server-name',
                            }
                        }
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                    json={'server': active_server},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json={'networks': []},
                ),
            ]
        )

        self.cloud.create_server(
            'server-name',
            image_id,
            'vanilla',
            nics=[{'net-id': 'some-network'}],
            wait=False,
        )

        self.assert_calls()

    def test_create_server_nics_port_id(self):
        '''Verify port-id in nics input turns into port in REST.'''
        build_server = fakes.make_fake_server('1234', '', 'BUILD')
        active_server = fakes.make_fake_server('1234', '', 'BUILD')
        image_id = uuid.uuid4().hex
        port_id = uuid.uuid4().hex

        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers']
                    ),
                    json={'server': build_server},
                    validate=dict(
                        json={
                            'server': {
                                'flavorRef': fakes.FLAVOR_ID,
                                'imageRef': image_id,
                                'max_count': 1,
                                'min_count': 1,
                                'networks': [{'port': port_id}],
                                'name': 'server-name',
                            }
                        }
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                    json={'server': active_server},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json={'networks': []},
                ),
            ]
        )

        self.cloud.create_server(
            'server-name',
            dict(id=image_id),
            dict(id=fakes.FLAVOR_ID),
            nics=[{'port-id': port_id}],
            wait=False,
        )

        self.assert_calls()

    def test_create_boot_attach_volume(self):
        build_server = fakes.make_fake_server('1234', '', 'BUILD')
        active_server = fakes.make_fake_server('1234', '', 'BUILD')

        vol = {
            'id': 'volume001',
            'status': 'available',
            'name': '',
            'attachments': [],
        }
        volume = meta.obj_to_munch(fakes.FakeVolume(**vol))

        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json={'networks': []},
                ),
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers']
                    ),
                    json={'server': build_server},
                    validate=dict(
                        json={
                            'server': {
                                'flavorRef': 'flavor-id',
                                'imageRef': 'image-id',
                                'max_count': 1,
                                'min_count': 1,
                                'block_device_mapping_v2': [
                                    {
                                        'boot_index': 0,
                                        'delete_on_termination': True,
                                        'destination_type': 'local',
                                        'source_type': 'image',
                                        'uuid': 'image-id',
                                    },
                                    {
                                        'boot_index': '-1',
                                        'delete_on_termination': False,
                                        'destination_type': 'volume',
                                        'source_type': 'volume',
                                        'uuid': 'volume001',
                                    },
                                ],
                                'name': 'server-name',
                                'networks': 'auto',
                            }
                        }
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                    json={'server': active_server},
                ),
            ]
        )

        self.cloud.create_server(
            name='server-name',
            image=dict(id='image-id'),
            flavor=dict(id='flavor-id'),
            boot_from_volume=False,
            volumes=[volume],
            wait=False,
        )

        self.assert_calls()

    def test_create_boot_from_volume_image_terminate(self):
        build_server = fakes.make_fake_server('1234', '', 'BUILD')
        active_server = fakes.make_fake_server('1234', '', 'BUILD')

        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json={'networks': []},
                ),
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers']
                    ),
                    json={'server': build_server},
                    validate=dict(
                        json={
                            'server': {
                                'flavorRef': 'flavor-id',
                                'imageRef': '',
                                'max_count': 1,
                                'min_count': 1,
                                'block_device_mapping_v2': [
                                    {
                                        'boot_index': '0',
                                        'delete_on_termination': True,
                                        'destination_type': 'volume',
                                        'source_type': 'image',
                                        'uuid': 'image-id',
                                        'volume_size': '1',
                                    }
                                ],
                                'name': 'server-name',
                                'networks': 'auto',
                            }
                        }
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                    json={'server': active_server},
                ),
            ]
        )

        self.cloud.create_server(
            name='server-name',
            image=dict(id='image-id'),
            flavor=dict(id='flavor-id'),
            boot_from_volume=True,
            terminate_volume=True,
            volume_size=1,
            wait=False,
        )

        self.assert_calls()

    def test_create_server_scheduler_hints(self):
        """
        Test that setting scheduler_hints will include them in POST request
        """
        scheduler_hints = {
            'group': self.getUniqueString('group'),
        }
        fake_server = fakes.make_fake_server('1234', '', 'BUILD')
        fake_server['scheduler_hints'] = scheduler_hints

        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json={'networks': []},
                ),
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers']
                    ),
                    json={'server': fake_server},
                    validate=dict(
                        json={
                            'server': {
                                'flavorRef': 'flavor-id',
                                'imageRef': 'image-id',
                                'max_count': 1,
                                'min_count': 1,
                                'name': 'server-name',
                                'networks': 'auto',
                            },
                            'OS-SCH-HNT:scheduler_hints': scheduler_hints,
                        }
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                    json={'server': fake_server},
                ),
            ]
        )

        self.cloud.create_server(
            name='server-name',
            image=dict(id='image-id'),
            flavor=dict(id='flavor-id'),
            scheduler_hints=scheduler_hints,
            wait=False,
        )

        self.assert_calls()

    def test_create_server_scheduler_hints_group_merge(self):
        """
        Test that setting both scheduler_hints and group results in merged
        hints in POST request
        """
        group_id = uuid.uuid4().hex
        group_name = self.getUniqueString('server-group')
        policies = ['affinity']
        fake_group = fakes.make_fake_server_group(
            group_id, group_name, policies
        )

        # The scheduler hints we pass in
        scheduler_hints = {
            'different_host': [],
        }

        # The scheduler hints we expect to be in POST request
        scheduler_hints_merged = {
            'different_host': [],
            'group': group_id,
        }

        fake_server = fakes.make_fake_server('1234', '', 'BUILD')
        fake_server['scheduler_hints'] = scheduler_hints_merged

        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['os-server-groups']
                    ),
                    json={'server_groups': [fake_group]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json={'networks': []},
                ),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers']
                    ),
                    json={'server': fake_server},
                    validate=dict(
                        json={
                            'server': {
                                'flavorRef': 'flavor-id',
                                'imageRef': 'image-id',
                                'max_count': 1,
                                'min_count': 1,
                                'name': 'server-name',
                                'networks': 'auto',
                            },
                            'OS-SCH-HNT:scheduler_hints': scheduler_hints_merged,  # noqa: E501
                        }
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                    json={'server': fake_server},
                ),
            ]
        )

        self.cloud.create_server(
            name='server-name',
            image=dict(id='image-id'),
            flavor=dict(id='flavor-id'),
            scheduler_hints=dict(scheduler_hints),
            group=group_name,
            wait=False,
        )

        self.assert_calls()

    def test_create_server_scheduler_hints_group_override(self):
        """
        Test that setting group in both scheduler_hints and group param prefers
        param
        """
        group_id_scheduler_hints = uuid.uuid4().hex
        group_id_param = uuid.uuid4().hex
        group_name = self.getUniqueString('server-group')
        policies = ['affinity']
        fake_group = fakes.make_fake_server_group(
            group_id_param, group_name, policies
        )

        # The scheduler hints we pass in that are expected to be ignored in
        # POST call
        scheduler_hints = {
            'group': group_id_scheduler_hints,
        }

        # The scheduler hints we expect to be in POST request
        group_scheduler_hints = {
            'group': group_id_param,
        }

        fake_server = fakes.make_fake_server('1234', '', 'BUILD')
        fake_server['scheduler_hints'] = group_scheduler_hints

        self.register_uris(
            [
                self.get_nova_discovery_mock_dict(),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['os-server-groups']
                    ),
                    json={'server_groups': [fake_group]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'network', 'public', append=['v2.0', 'networks']
                    ),
                    json={'networks': []},
                ),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers']
                    ),
                    json={'server': fake_server},
                    validate=dict(
                        json={
                            'server': {
                                'flavorRef': 'flavor-id',
                                'imageRef': 'image-id',
                                'max_count': 1,
                                'min_count': 1,
                                'name': 'server-name',
                                'networks': 'auto',
                            },
                            'OS-SCH-HNT:scheduler_hints': group_scheduler_hints,  # noqa: E501
                        }
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'compute', 'public', append=['servers', '1234']
                    ),
                    json={'server': fake_server},
                ),
            ]
        )

        self.cloud.create_server(
            name='server-name',
            image=dict(id='image-id'),
            flavor=dict(id='flavor-id'),
            scheduler_hints=dict(scheduler_hints),
            group=group_name,
            wait=False,
        )

        self.assert_calls()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.clustering.v1 import profile
from openstack.tests.unit import base


FAKE_ID = '9b127538-a675-4271-ab9b-f24f54cfe173'
FAKE_NAME = 'test_profile'

FAKE = {
    'metadata': {},
    'name': FAKE_NAME,
    'id': FAKE_ID,
    'spec': {
        'type': 'os.nova.server',
        'version': 1.0,
        'properties': {
            'flavor': 1,
            'image': 'cirros-0.3.2-x86_64-uec',
            'key_name': 'oskey',
            'name': 'cirros_server',
        },
    },
    'project': '42d9e9663331431f97b75e25136307ff',
    'domain': '204ccccd267b40aea871750116b5b184',
    'user': '3747afc360b64702a53bdd64dc1b8976',
    'type': 'os.nova.server',
    'created_at': '2015-10-10T12:46:36.000000',
    'updated_at': '2016-10-10T12:46:36.000000',
}


class TestProfile(base.TestCase):
    def setUp(self):
        super(TestProfile, self).setUp()

    def test_basic(self):
        sot = profile.Profile()
        self.assertEqual('profile', sot.resource_key)
        self.assertEqual('profiles', sot.resources_key)
        self.assertEqual('/profiles', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertEqual('PATCH', sot.commit_method)

    def test_instantiate(self):
        sot = profile.Profile(**FAKE)
        self.assertEqual(FAKE['id'], sot.id)
        self.assertEqual(FAKE['name'], sot.name)
        self.assertEqual(FAKE['metadata'], sot.metadata)
        self.assertEqual(FAKE['spec'], sot.spec)
        self.assertEqual(FAKE['project'], sot.project_id)
        self.assertEqual(FAKE['domain'], sot.domain_id)
        self.assertEqual(FAKE['user'], sot.user_id)
        self.assertEqual(FAKE['type'], sot.type)
        self.assertEqual(FAKE['created_at'], sot.created_at)
        self.assertEqual(FAKE['updated_at'], sot.updated_at)


class TestProfileValidate(base.TestCase):
    def setUp(self):
        super(TestProfileValidate, self).setUp()

    def test_basic(self):
        sot = profile.ProfileValidate()
        self.assertEqual('profile', sot.resource_key)
        self.assertEqual('profiles', sot.resources_key)
        self.assertEqual('/profiles/validate', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_list)
        self.assertEqual('PUT', sot.commit_method)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class Profile(resource.Resource):
    resource_key = 'profile'
    resources_key = 'profiles'
    base_path = '/profiles'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    commit_method = 'PATCH'

    _query_mapping = resource.QueryParameters(
        'sort', 'global_project', 'type', 'name'
    )

    # Bodyerties
    #: The name of the profile
    name = resource.Body('name')
    #: The type of the profile.
    type = resource.Body('type')
    #: The ID of the project this profile belongs to.
    project_id = resource.Body('project')
    #: The domain ID of the profile.
    domain_id = resource.Body('domain')
    #: The ID of the user who created this profile.
    user_id = resource.Body('user')
    #: The spec of the profile.
    spec = resource.Body('spec', type=dict)
    #: A collection of key-value pairs that are attached to the profile.
    metadata = resource.Body('metadata', type=dict)
    #: Timestamp of when the profile was created.
    created_at = resource.Body('created_at')
    #: Timestamp of when the profile was last updated.
    updated_at = resource.Body('updated_at')


class ProfileValidate(Profile):
    base_path = '/profiles/validate'
    allow_create = True
    allow_fetch = False
    allow_commit = False
    allow_delete = False
    allow_list = False

    commit_method = 'PUT'



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from openstack import exceptions
from openstack.tests.functional.baremetal import base


class TestBareMetalVolumeconnector(base.BaseBaremetalTest):
    min_microversion = '1.32'

    def setUp(self):
        super(TestBareMetalVolumeconnector, self).setUp()
        self.node = self.create_node(provision_state='enroll')

    def test_volume_connector_create_get_delete(self):
        self.conn.baremetal.set_node_provision_state(
            self.node, 'manage', wait=True
        )
        self.conn.baremetal.set_node_power_state(self.node, 'power off')
        volume_connector = self.create_volume_connector(
            connector_id='iqn.2017-07.org.openstack:01:d9a51732c3f', type='iqn'
        )

        loaded = self.conn.baremetal.get_volume_connector(volume_connector.id)
        self.assertEqual(loaded.id, volume_connector.id)
        self.assertIsNotNone(loaded.node_id)

        with_fields = self.conn.baremetal.get_volume_connector(
            volume_connector.id, fields=['uuid', 'extra']
        )
        self.assertEqual(volume_connector.id, with_fields.id)
        self.assertIsNone(with_fields.node_id)

        self.conn.baremetal.delete_volume_connector(
            volume_connector, ignore_missing=False
        )
        self.assertRaises(
            exceptions.ResourceNotFound,
            self.conn.baremetal.get_volume_connector,
            volume_connector.id,
        )

    def test_volume_connector_list(self):
        node2 = self.create_node(name='test-node')
        self.conn.baremetal.set_node_provision_state(
            node2, 'manage', wait=True
        )
        self.conn.baremetal.set_node_power_state(node2, 'power off')
        self.conn.baremetal.set_node_provision_state(
            self.node, 'manage', wait=True
        )
        self.conn.baremetal.set_node_power_state(self.node, 'power off')
        vc1 = self.create_volume_connector(
            connector_id='iqn.2018-07.org.openstack:01:d9a514g2c32',
            node_id=node2.id,
            type='iqn',
        )
        vc2 = self.create_volume_connector(
            connector_id='iqn.2017-07.org.openstack:01:d9a51732c4g',
            node_id=self.node.id,
            type='iqn',
        )

        vcs = self.conn.baremetal.volume_connectors(node=self.node.id)
        self.assertEqual([v.id for v in vcs], [vc2.id])

        vcs = self.conn.baremetal.volume_connectors(node=node2.id)
        self.assertEqual([v.id for v in vcs], [vc1.id])

        vcs = self.conn.baremetal.volume_connectors(node='test-node')
        self.assertEqual([v.id for v in vcs], [vc1.id])

    def test_volume_connector_list_update_delete(self):
        self.conn.baremetal.set_node_provision_state(
            self.node, 'manage', wait=True
        )
        self.conn.baremetal.set_node_power_state(self.node, 'power off')
        self.create_volume_connector(
            connector_id='iqn.2020-07.org.openstack:02:d9451472ce2',
            node_id=self.node.id,
            type='iqn',
            extra={'foo': 'bar'},
        )
        volume_connector = next(
            self.conn.baremetal.volume_connectors(
                details=True, node=self.node.id
            )
        )
        self.assertEqual(volume_connector.extra, {'foo': 'bar'})

        # This test checks that resources returned from listing are usable
        self.conn.baremetal.update_volume_connector(
            volume_connector, extra={'foo': 42}
        )
        self.conn.baremetal.delete_volume_connector(
            volume_connector, ignore_missing=False
        )

    def test_volume_connector_update(self):
        self.conn.baremetal.set_node_provision_state(
            self.node, 'manage', wait=True
        )
        self.conn.baremetal.set_node_power_state(self.node, 'power off')
        volume_connector = self.create_volume_connector(
            connector_id='iqn.2019-07.org.openstack:03:de45b472c40',
            node_id=self.node.id,
            type='iqn',
        )
        volume_connector.extra = {'answer': 42}

        volume_connector = self.conn.baremetal.update_volume_connector(
            volume_connector
        )
        self.assertEqual({'answer': 42}, volume_connector.extra)

        volume_connector = self.conn.baremetal.get_volume_connector(
            volume_connector.id
        )
        self.assertEqual({'answer': 42}, volume_connector.extra)

    def test_volume_connector_patch(self):
        vol_conn_id = 'iqn.2020-07.org.openstack:04:de45b472c40'
        self.conn.baremetal.set_node_provision_state(
            self.node, 'manage', wait=True
        )
        self.conn.baremetal.set_node_power_state(self.node, 'power off')
        volume_connector = self.create_volume_connector(
            connector_id=vol_conn_id, node_id=self.node.id, type='iqn'
        )

        volume_connector = self.conn.baremetal.patch_volume_connector(
            volume_connector, dict(path='/extra/answer', op='add', value=42)
        )
        self.assertEqual({'answer': 42}, volume_connector.extra)
        self.assertEqual(vol_conn_id, volume_connector.connector_id)

        volume_connector = self.conn.baremetal.get_volume_connector(
            volume_connector.id
        )
        self.assertEqual({'answer': 42}, volume_connector.extra)

    def test_volume_connector_negative_non_existing(self):
        uuid = "5c9dcd04-2073-49bc-9618-99ae634d8971"
        self.assertRaises(
            exceptions.ResourceNotFound,
            self.conn.baremetal.get_volume_connector,
            uuid,
        )
        self.assertRaises(
            exceptions.ResourceNotFound,
            self.conn.baremetal.find_volume_connector,
            uuid,
            ignore_missing=False,
        )
        self.assertRaises(
            exceptions.ResourceNotFound,
            self.conn.baremetal.delete_volume_connector,
            uuid,
            ignore_missing=False,
        )
        self.assertIsNone(self.conn.baremetal.find_volume_connector(uuid))
        self.assertIsNone(self.conn.baremetal.delete_volume_connector(uuid))

    def test_volume_connector_fields(self):
        self.create_node()
        self.conn.baremetal.set_node_provision_state(
            self.node, 'manage', wait=True
        )
        self.conn.baremetal.set_node_power_state(self.node, 'power off')
        self.create_volume_connector(
            connector_id='iqn.2018-08.org.openstack:04:de45f37c48',
            node_id=self.node.id,
            type='iqn',
        )
        result = self.conn.baremetal.volume_connectors(
            fields=['uuid', 'node_id']
        )
        for item in result:
            self.assertIsNotNone(item.id)
            self.assertIsNone(item.connector_id)



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
test_port
----------------------------------

Functional tests for port resource.
"""

import random
import string

from openstack import exceptions
from openstack.tests.functional import base


class TestPort(base.BaseFunctionalTest):
    def setUp(self):
        super(TestPort, self).setUp()
        # Skip Neutron tests if neutron is not present
        if not self.user_cloud.has_service('network'):
            self.skipTest('Network service not supported by cloud')

        net_name = self.getUniqueString('CloudPortName')
        self.net = self.user_cloud.network.create_network(name=net_name)
        self.addCleanup(self.user_cloud.network.delete_network, self.net.id)

        # Generate a unique port name to allow concurrent tests
        self.new_port_name = 'test_' + ''.join(
            random.choice(string.ascii_lowercase) for _ in range(5)
        )

        self.addCleanup(self._cleanup_ports)

    def _cleanup_ports(self):
        exception_list = list()

        for p in self.user_cloud.list_ports():
            if p['name'].startswith(self.new_port_name):
                try:
                    self.user_cloud.delete_port(name_or_id=p['id'])
                except Exception as e:
                    # We were unable to delete this port, let's try with next
                    exception_list.append(str(e))
                    continue

        if exception_list:
            # Raise an error: we must make users aware that something went
            # wrong
            raise exceptions.SDKException('\n'.join(exception_list))

    def test_create_port(self):
        port_name = self.new_port_name + '_create'

        port = self.user_cloud.create_port(
            network_id=self.net.id, name=port_name
        )
        self.assertIsInstance(port, dict)
        self.assertIn('id', port)
        self.assertEqual(port.get('name'), port_name)

    def test_get_port(self):
        port_name = self.new_port_name + '_get'

        port = self.user_cloud.create_port(
            network_id=self.net.id, name=port_name
        )
        self.assertIsInstance(port, dict)
        self.assertIn('id', port)
        self.assertEqual(port.get('name'), port_name)

        updated_port = self.user_cloud.get_port(name_or_id=port['id'])
        # extra_dhcp_opts is added later by Neutron...
        if 'extra_dhcp_opts' in updated_port and 'extra_dhcp_opts' not in port:
            del updated_port['extra_dhcp_opts']
        self.assertEqual(port, updated_port)

    def test_get_port_by_id(self):
        port_name = self.new_port_name + '_get_by_id'

        port = self.user_cloud.create_port(
            network_id=self.net.id, name=port_name
        )
        self.assertIsInstance(port, dict)
        self.assertIn('id', port)
        self.assertEqual(port.get('name'), port_name)

        updated_port = self.user_cloud.get_port_by_id(port['id'])
        # extra_dhcp_opts is added later by Neutron...
        if 'extra_dhcp_opts' in updated_port and 'extra_dhcp_opts' not in port:
            del updated_port['extra_dhcp_opts']
        self.assertEqual(port, updated_port)

    def test_update_port(self):
        port_name = self.new_port_name + '_update'
        new_port_name = port_name + '_new'

        self.user_cloud.create_port(network_id=self.net.id, name=port_name)

        port = self.user_cloud.update_port(
            name_or_id=port_name, name=new_port_name
        )
        self.assertIsInstance(port, dict)
        self.assertEqual(port.get('name'), new_port_name)

        updated_port = self.user_cloud.get_port(name_or_id=port['id'])
        self.assertEqual(port.get('name'), new_port_name)
        port.pop('revision_number', None)
        port.pop(u'revision_number', None)
        port.pop('updated_at', None)
        port.pop(u'updated_at', None)
        updated_port.pop('revision_number', None)
        updated_port.pop(u'revision_number', None)
        updated_port.pop('updated_at', None)
        updated_port.pop(u'updated_at', None)

        self.assertEqual(port, updated_port)

    def test_delete_port(self):
        port_name = self.new_port_name + '_delete'

        port = self.user_cloud.create_port(
            network_id=self.net.id, name=port_name
        )
        self.assertIsInstance(port, dict)
        self.assertIn('id', port)
        self.assertEqual(port.get('name'), port_name)

        updated_port = self.user_cloud.get_port(name_or_id=port['id'])
        self.assertIsNotNone(updated_port)

        self.user_cloud.delete_port(name_or_id=port_name)

        updated_port = self.user_cloud.get_port(name_or_id=port['id'])
        self.assertIsNone(updated_port)



# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from openstack import exceptions
from openstack.tests.unit import base


class TestVolumeAccess(base.TestCase):
    def setUp(self):
        super(TestVolumeAccess, self).setUp()
        self.use_cinder()

    def test_list_volume_types(self):
        volume_type = dict(
            id='voltype01',
            description='volume type description',
            name='name',
            is_public=False,
        )
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['types']
                    ),
                    json={'volume_types': [volume_type]},
                )
            ]
        )
        self.assertTrue(self.cloud.list_volume_types())
        self.assert_calls()

    def test_get_volume_type(self):
        volume_type = dict(
            id='voltype01',
            description='volume type description',
            name='name',
            is_public=False,
        )
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['types']
                    ),
                    json={'volume_types': [volume_type]},
                )
            ]
        )
        volume_type_got = self.cloud.get_volume_type(volume_type['name'])
        self.assertEqual(volume_type_got.id, volume_type['id'])

    def test_get_volume_type_access(self):
        volume_type = dict(
            id='voltype01',
            description='volume type description',
            name='name',
            is_public=False,
        )
        volume_type_access = [
            dict(volume_type_id='voltype01', name='name', project_id='prj01'),
            dict(volume_type_id='voltype01', name='name', project_id='prj02'),
        ]
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['types']
                    ),
                    json={'volume_types': [volume_type]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3',
                        'public',
                        append=[
                            'types',
                            volume_type['id'],
                            'os-volume-type-access',
                        ],
                    ),
                    json={'volume_type_access': volume_type_access},
                ),
            ]
        )
        self.assertEqual(
            len(self.cloud.get_volume_type_access(volume_type['name'])), 2
        )
        self.assert_calls()

    def test_remove_volume_type_access(self):
        volume_type = dict(
            id='voltype01',
            description='volume type description',
            name='name',
            is_public=False,
        )
        project_001 = dict(
            volume_type_id='voltype01', name='name', project_id='prj01'
        )
        project_002 = dict(
            volume_type_id='voltype01', name='name', project_id='prj02'
        )
        volume_type_access = [project_001, project_002]
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['types']
                    ),
                    json={'volume_types': [volume_type]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3',
                        'public',
                        append=[
                            'types',
                            volume_type['id'],
                            'os-volume-type-access',
                        ],
                    ),
                    json={'volume_type_access': volume_type_access},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['types']
                    ),
                    json={'volume_types': [volume_type]},
                ),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'volumev3',
                        'public',
                        append=['types', volume_type['id'], 'action'],
                    ),
                    json={
                        'removeProjectAccess': {
                            'project': project_001['project_id']
                        }
                    },
                    validate=dict(
                        json={
                            'removeProjectAccess': {
                                'project': project_001['project_id']
                            }
                        }
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['types']
                    ),
                    json={'volume_types': [volume_type]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3',
                        'public',
                        append=[
                            'types',
                            volume_type['id'],
                            'os-volume-type-access',
                        ],
                    ),
                    json={'volume_type_access': [project_001]},
                ),
            ]
        )
        self.assertEqual(
            len(self.cloud.get_volume_type_access(volume_type['name'])), 2
        )
        self.cloud.remove_volume_type_access(
            volume_type['name'], project_001['project_id']
        )
        self.assertEqual(
            len(self.cloud.get_volume_type_access(volume_type['name'])), 1
        )
        self.assert_calls()

    def test_add_volume_type_access(self):
        volume_type = dict(
            id='voltype01',
            description='volume type description',
            name='name',
            is_public=False,
        )
        project_001 = dict(
            volume_type_id='voltype01', name='name', project_id='prj01'
        )
        project_002 = dict(
            volume_type_id='voltype01', name='name', project_id='prj02'
        )
        volume_type_access = [project_001, project_002]
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['types']
                    ),
                    json={'volume_types': [volume_type]},
                ),
                dict(
                    method='POST',
                    uri=self.get_mock_url(
                        'volumev3',
                        'public',
                        append=['types', volume_type['id'], 'action'],
                    ),
                    json={
                        'addProjectAccess': {
                            'project': project_002['project_id']
                        }
                    },
                    validate=dict(
                        json={
                            'addProjectAccess': {
                                'project': project_002['project_id']
                            }
                        }
                    ),
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['types']
                    ),
                    json={'volume_types': [volume_type]},
                ),
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3',
                        'public',
                        append=[
                            'types',
                            volume_type['id'],
                            'os-volume-type-access',
                        ],
                    ),
                    json={'volume_type_access': volume_type_access},
                ),
            ]
        )
        self.cloud.add_volume_type_access(
            volume_type['name'], project_002['project_id']
        )
        self.assertEqual(
            len(self.cloud.get_volume_type_access(volume_type['name'])), 2
        )
        self.assert_calls()

    def test_add_volume_type_access_missing(self):
        volume_type = dict(
            id='voltype01',
            description='volume type description',
            name='name',
            is_public=False,
        )
        project_001 = dict(
            volume_type_id='voltype01', name='name', project_id='prj01'
        )
        self.register_uris(
            [
                dict(
                    method='GET',
                    uri=self.get_mock_url(
                        'volumev3', 'public', append=['types']
                    ),
                    json={'volume_types': [volume_type]},
                )
            ]
        )
        with testtools.ExpectedException(
            exceptions.SDKException,
            "VolumeType not found: MISSING",
        ):
            self.cloud.add_volume_type_access(
                "MISSING", project_001['project_id']
            )
        self.assert_calls()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class SfcFlowClassifier(resource.Resource):
    resource_key = 'flow_classifier'
    resources_key = 'flow_classifiers'
    base_path = '/sfc/flow_classifiers'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'description',
        'name',
        'project_id',
        'tenant_id',
        'ethertype',
        'protocol',
        'source_port_range_min',
        'source_port_range_max',
        'destination_port_range_min',
        'destination_port_range_max',
        'logical_source_port',
        'logical_destination_port',
    )

    # Properties
    #: Human-readable description for the resource.
    description = resource.Body('description')
    #: Human-readable name of the resource. Default is an empty string.
    name = resource.Body('name')
    #: Must be IPv4 or IPv6, and addresses represented in CIDR must match
    # the ingress or egress rules.
    ethertype = resource.Body('ethertype')
    #: The IP protocol can be represented by a string, an integer, or null.
    #: Valid values: any (0), ah (51), dccp (33), egp (8), esp (50), gre (47),
    #: icmp (1), icmpv6 (58), igmp (2), ipip (4), ipv6-encap (41),
    #: ipv6-frag (44), ipv6-icmp (58), ipv6-nonxt (59), ipv6-opts (60),
    #: ipv6-route (43), ospf (89), pgm (113), rsvp (46), sctp (132), tcp (6),
    #: udp (17), udplite (136), vrrp (112).
    protocol = resource.Body('protocol')
    #: Minimum source protocol port.
    source_port_range_min = resource.Body('source_port_range_min', type=int)
    #: Maximum source protocol port.
    source_port_range_max = resource.Body('source_port_range_max', type=int)
    #: Minimum destination protocol port.
    destination_port_range_min = resource.Body(
        'destination_port_range_min', type=int
    )
    #: Maximum destination protocol port.
    destination_port_range_max = resource.Body(
        'destination_port_range_max', type=int
    )
    #: The source IP prefix.
    source_ip_prefix = resource.Body('source_ip_prefix')
    #: The destination IP prefix.
    destination_ip_prefix = resource.Body('destination_ip_prefix')
    #: The UUID of the source logical port.
    logical_source_port = resource.Body('logical_source_port')
    #: The UUID of the destination logical port.
    logical_destination_port = resource.Body('logical_destination_port')
    #: A dictionary of L7 parameters, in the form of
    #: logical_source_network: uuid, logical_destination_network: uuid.
    l7_parameters = resource.Body('l7_parameters', type=dict)
    #: Summary field of a Flow Classifier, composed of the
    #: protocol, source protcol port, destination ptocolo port,
    #: logical_source_port, logical_destination_port and
    #: l7_parameters
    summary = resource.Computed('summary', default='')
    project_id = resource.Body('project_id', alias='tenant_id')
    #: Tenant_id (deprecated attribute).
    tenant_id = resource.Body('tenant_id', deprecated=True)



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
test_endpoint
----------------------------------

Functional tests for endpoint resource.
"""

import random
import string

from openstack.cloud.exc import OpenStackCloudUnavailableFeature
from openstack import exceptions
from openstack.tests.functional import base


class TestEndpoints(base.KeystoneBaseFunctionalTest):
    endpoint_attributes = [
        'id',
        'region',
        'publicurl',
        'internalurl',
        'service_id',
        'adminurl',
    ]

    def setUp(self):
        super(TestEndpoints, self).setUp()
        if not self.operator_cloud:
            self.skipTest("Operator cloud is required for this test")

        # Generate a random name for services and regions in this test
        self.new_item_name = 'test_' + ''.join(
            random.choice(string.ascii_lowercase) for _ in range(5)
        )

        self.addCleanup(self._cleanup_services)
        self.addCleanup(self._cleanup_endpoints)

    def _cleanup_endpoints(self):
        exception_list = list()
        for e in self.operator_cloud.list_endpoints():
            if e.get('region') is not None and e['region'].startswith(
                self.new_item_name
            ):
                try:
                    self.operator_cloud.delete_endpoint(id=e['id'])
                except Exception as e:
                    # We were unable to delete a service, let's try with next
                    exception_list.append(str(e))
                    continue
        if exception_list:
            # Raise an error: we must make users aware that something went
            # wrong
            raise exceptions.SDKException('\n'.join(exception_list))

    def _cleanup_services(self):
        exception_list = list()
        for s in self.operator_cloud.list_services():
            if s['name'] is not None and s['name'].startswith(
                self.new_item_name
            ):
                try:
                    self.operator_cloud.delete_service(name_or_id=s['id'])
                except Exception as e:
                    # We were unable to delete a service, let's try with next
                    exception_list.append(str(e))
                    continue
        if exception_list:
            # Raise an error: we must make users aware that something went
            # wrong
            raise exceptions.SDKException('\n'.join(exception_list))

    def test_create_endpoint(self):
        service_name = self.new_item_name + '_create'

        region = list(self.operator_cloud.identity.regions())[0].id

        service = self.operator_cloud.create_service(
            name=service_name,
            type='test_type',
            description='this is a test description',
        )

        endpoints = self.operator_cloud.create_endpoint(
            service_name_or_id=service['id'],
            public_url='http://public.test/',
            internal_url='http://internal.test/',
            admin_url='http://admin.url/',
            region=region,
        )

        self.assertNotEqual([], endpoints)
        self.assertIsNotNone(endpoints[0].get('id'))

        # Test None parameters
        endpoints = self.operator_cloud.create_endpoint(
            service_name_or_id=service['id'],
            public_url='http://public.test/',
            region=region,
        )

        self.assertNotEqual([], endpoints)
        self.assertIsNotNone(endpoints[0].get('id'))

    def test_update_endpoint(self):
        ver = self.operator_cloud.config.get_api_version('identity')
        if ver.startswith('2'):
            # NOTE(SamYaple): Update endpoint only works with v3 api
            self.assertRaises(
                OpenStackCloudUnavailableFeature,
                self.operator_cloud.update_endpoint,
                'endpoint_id1',
            )
        else:
            # service operations require existing region. Do not test updating
            # region for now
            region = list(self.operator_cloud.identity.regions())[0].id

            service = self.operator_cloud.create_service(
                name='service1', type='test_type'
            )
            endpoint = self.operator_cloud.create_endpoint(
                service_name_or_id=service['id'],
                url='http://admin.url/',
                interface='admin',
                region=region,
                enabled=False,
            )[0]

            new_service = self.operator_cloud.create_service(
                name='service2', type='test_type'
            )
            new_endpoint = self.operator_cloud.update_endpoint(
                endpoint.id,
                service_name_or_id=new_service.id,
                url='http://public.url/',
                interface='public',
                region=region,
                enabled=True,
            )

            self.assertEqual(new_endpoint.url, 'http://public.url/')
            self.assertEqual(new_endpoint.interface, 'public')
            self.assertEqual(new_endpoint.region_id, region)
            self.assertEqual(new_endpoint.service_id, new_service.id)
            self.assertTrue(new_endpoint.is_enabled)

    def test_list_endpoints(self):
        service_name = self.new_item_name + '_list'

        region = list(self.operator_cloud.identity.regions())[0].id

        service = self.operator_cloud.create_service(
            name=service_name,
            type='test_type',
            description='this is a test description',
        )

        endpoints = self.operator_cloud.create_endpoint(
            service_name_or_id=service['id'],
            public_url='http://public.test/',
            internal_url='http://internal.test/',
            region=region,
        )

        observed_endpoints = self.operator_cloud.list_endpoints()
        found = False
        for e in observed_endpoints:
            # Test all attributes are returned
            for endpoint in endpoints:
                if e['id'] == endpoint['id']:
                    found = True
                    self.assertEqual(service['id'], e['service_id'])
                    if 'interface' in e:
                        if e['interface'] == 'internal':
                            self.assertEqual('http://internal.test/', e['url'])
                        elif e['interface'] == 'public':
                            self.assertEqual('http://public.test/', e['url'])
                    else:
                        self.assertEqual('http://public.test/', e['publicurl'])
                        self.assertEqual(
                            'http://internal.test/', e['internalurl']
                        )
                    self.assertEqual(region, e['region_id'])

        self.assertTrue(found, msg='new endpoint not found in endpoints list!')

    def test_delete_endpoint(self):
        service_name = self.new_item_name + '_delete'

        region = list(self.operator_cloud.identity.regions())[0].id

        service = self.operator_cloud.create_service(
            name=service_name,
            type='test_type',
            description='this is a test description',
        )

        endpoints = self.operator_cloud.create_endpoint(
            service_name_or_id=service['id'],
            public_url='http://public.test/',
            internal_url='http://internal.test/',
            region=region,
        )

        self.assertNotEqual([], endpoints)
        for endpoint in endpoints:
            self.operator_cloud.delete_endpoint(endpoint['id'])

        observed_endpoints = self.operator_cloud.list_endpoints()
        found = False
        for e in observed_endpoints:
            for endpoint in endpoints:
                if e['id'] == endpoint['id']:
                    found = True
                    break
        self.assertEqual(False, found, message='new endpoint was not deleted!')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.common import metadata
from openstack import exceptions
from openstack import resource
from openstack import utils


class Share(resource.Resource, metadata.MetadataMixin):
    resource_key = "share"
    resources_key = "shares"
    base_path = "/shares"

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_list = True
    allow_head = False
    allow_delete = True

    #: Properties
    #: The share instance access rules status. A valid value is active,
    #: error, or syncing.
    access_rules_status = resource.Body("access_rules_status", type=str)
    #: The availability zone.
    availability_zone = resource.Body("availability_zone", type=str)
    #: The date and time stamp when the resource was created within the
    #: service’s database.
    created_at = resource.Body("created_at", type=str)
    #: The user defined description of the resource.
    description = resource.Body("description", type=str)
    #: The share host name.
    host = resource.Body("host", type=str)
    #: The level of visibility for the share.
    is_public = resource.Body("is_public", type=bool)
    #: Whether or not this share supports snapshots that can be
    #: cloned into new shares.
    is_creating_new_share_from_snapshot_supported = resource.Body(
        "create_share_from_snapshot_support", type=bool
    )
    #: Whether the share's snapshots can be mounted directly and access
    #: controlled independently or not.
    is_mounting_snapshot_supported = resource.Body(
        "mount_snapshot_support", type=bool
    )
    #: Whether the share can be reverted to its latest snapshot or not.
    is_reverting_to_snapshot_supported = resource.Body(
        "revert_to_snapshot_support", type=bool
    )
    #: An extra specification that filters back ends by whether the share
    #: supports snapshots or not.
    is_snapshot_supported = resource.Body("snapshot_support", type=bool)
    #: Indicates whether the share has replicas or not.
    is_replicated = resource.Body("has_replicas", type=bool)
    #: One or more metadata key and value pairs as a dictionary of strings.
    metadata = resource.Body("metadata", type=dict)
    #: The progress of the share creation.
    progress = resource.Body("progress", type=str)
    #: The ID of the project that owns the resource.
    project_id = resource.Body("project_id", type=str)
    #: The share replication type. Valid values are none, readable,
    #: writable and dr.
    replication_type = resource.Body("replication_type", type=str)
    #: The UUID of the share group that this shares belongs to.
    share_group_id = resource.Body("share_group_id", type=str)
    #: The share network ID.
    share_network_id = resource.Body("share_network_id", type=str)
    #: The Shared File Systems protocol. A valid value is NFS,
    #: CIFS, GlusterFS, HDFS, CephFS, MAPRFS
    share_protocol = resource.Body("share_proto", type=str)
    #: The UUID of the share server.
    share_server_id = resource.Body("share_server_id", type=str)
    #: The UUID of the share type. In minor versions, this parameter is a
    #: share type name, as a string.
    share_type = resource.Body("share_type", type=str)
    #: Name of the share type.
    share_type_name = resource.Body("share_type_name", type=str)
    #: The share size, in GiBs.
    size = resource.Body("size", type=int)
    #: The UUID of the snapshot that was used to create the
    #: share.
    snapshot_id = resource.Body("snapshot_id", type=str)
    #: The ID of the group snapshot instance that was used to create
    #: this share.
    source_share_group_snapshot_member_id = resource.Body(
        "source_share_group_snapshot_member_id", type=str
    )
    #: The share status
    status = resource.Body("status", type=str)
    #: For the share migration, the migration task state.
    task_state = resource.Body("task_state", type=str)
    #: ID of the user that the share was created by.
    user_id = resource.Body("user_id", type=str)
    #: Display name for updating name
    display_name = resource.Body("display_name", type=str)
    #: Display description for updating description
    display_description = resource.Body("display_description", type=str)

    def _action(self, session, body, action='patch', microversion=None):
        """Perform share instance actions given the message body"""
        url = utils.urljoin(self.base_path, self.id, 'action')
        headers = {'Accept': ''}

        if microversion is None:
            microversion = self._get_microversion(session, action=action)

        response = session.post(
            url, json=body, headers=headers, microversion=microversion
        )

        exceptions.raise_from_response(response)
        return response

    def extend_share(self, session, new_size, force=False):
        """Extend the share size.

        :param float new_size: The new size of the share
            in GiB.
        :param bool force: Whether or not to use force, bypassing
            the scheduler. Requires admin privileges. Defaults to False.
        :returns: The result of the action.
        :rtype: ``None``
        """

        extend_body = {"new_size": new_size}

        if force is True:
            extend_body['force'] = True

        body = {"extend": extend_body}
        self._action(session, body)

    def shrink_share(self, session, new_size):
        """Shrink the share size.

        :param float new_size: The new size of the share
            in GiB.
        :returns: ``None``
        """

        body = {"shrink": {'new_size': new_size}}
        self._action(session, body)

    def revert_to_snapshot(self, session, snapshot_id):
        """Revert the share to the given snapshot.

        :param str snapshot_id: The id of the snapshot to revert to.
        :returns: ``None``
        """
        body = {"revert": {"snapshot_id": snapshot_id}}
        self._action(session, body)

    def manage(self, session, protocol, export_path, service_host, **params):
        """Manage a share.

        :param session: A session object used for sending request.
        :param str protocol: The shared file systems protocol of this share.
        :param str export_path: The export path formatted according to the
            protocol.
        :param str service_host: The manage-share service host.
        :param kwargs params: Optional parameters to be sent. Available
            parameters include:

            * name: The user defined name of the resource.
            * share_type: The name or ID of the share type to be used to create
              the resource.
            * driver_options: A set of one or more key and value pairs, as a
              dictionary of strings, that describe driver options.
            * is_public: The level of visibility for the share.
            * description: The user defiend description of the resource.
            * share_server_id: The UUID of the share server.

        :returns: The share that was managed.
        """

        path = 'manage'
        attrs = {
            'share': {
                'protocol': protocol,
                'export_path': export_path,
                'service_host': service_host,
            }
        }

        attrs['share'].update(params)

        url = utils.urljoin(self.base_path, path)
        resp = session.post(url, json=attrs)

        self._translate_response(resp)
        return self

    def unmanage(self, session):
        """Unmanage a share.

        :param session: A session object used for sending request.
        :returns: ``None``
        """

        body = {'unmanage': None}

        self._action(session, body)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.common import tag
from openstack.network.v2 import _base
from openstack import resource


class FloatingIP(_base.NetworkResource, tag.TagMixin):
    name_attribute = "floating_ip_address"
    resource_name = "floating ip"
    resource_key = 'floatingip'
    resources_key = 'floatingips'
    base_path = '/floatingips'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    # For backward compatibility include tenant_id as query param
    _query_mapping = resource.QueryParameters(
        'description',
        'fixed_ip_address',
        'floating_ip_address',
        'floating_network_id',
        'port_id',
        'router_id',
        'status',
        'subnet_id',
        'project_id',
        'tenant_id',
        tenant_id='project_id',
        **tag.TagMixin._tag_query_parameters
    )

    # Properties
    #: Timestamp at which the floating IP was created.
    created_at = resource.Body('created_at')
    #: The floating IP description.
    description = resource.Body('description')
    #: The DNS domain.
    dns_domain = resource.Body('dns_domain')
    #: The DNS name.
    dns_name = resource.Body('dns_name')
    #: The fixed IP address associated with the floating IP. If you
    #: intend to associate the floating IP with a fixed IP at creation
    #: time, then you must indicate the identifier of the internal port.
    #: If an internal port has multiple associated IP addresses, the
    #: service chooses the first IP unless you explicitly specify the
    #: parameter fixed_ip_address to select a specific IP.
    fixed_ip_address = resource.Body('fixed_ip_address')
    #: The floating IP address.
    floating_ip_address = resource.Body('floating_ip_address')
    #: Floating IP object doesn't have name attribute, set ip address to name
    #: so that user could find floating IP by UUID or IP address using find_ip
    name = floating_ip_address
    #: The ID of the network associated with the floating IP.
    floating_network_id = resource.Body('floating_network_id')
    #: Read-only. The details of the port that this floating IP associates
    #: with. Present if ``fip-port-details`` extension is loaded.
    #: *Type: dict with keys: name, network_id, mac_address, admin_state_up,
    #: status, device_id, device_owner*
    port_details = resource.Body('port_details', type=dict)
    #: The port ID.
    port_id = resource.Body('port_id')
    #: The ID of the QoS policy attached to the floating IP.
    qos_policy_id = resource.Body('qos_policy_id')
    #: The ID of the project this floating IP is associated with.
    project_id = resource.Body('project_id', alias='tenant_id')
    #: Tenant_id (deprecated attribute).
    tenant_id = resource.Body('tenant_id', deprecated=True)
    #: The ID of an associated router.
    router_id = resource.Body('router_id')
    #: The floating IP status. Value is ``ACTIVE`` or ``DOWN``.
    status = resource.Body('status')
    #: Timestamp at which the floating IP was last updated.
    updated_at = resource.Body('updated_at')
    #: The Subnet ID associated with the floating IP.
    subnet_id = resource.Body('subnet_id')

    @classmethod
    def find_available(cls, session):
        # server-side filtering on empty values is not always supported.
        # TODO(mordred) Make this check for support for the server-side filter
        for ip in cls.list(session):
            if not ip.port_id:
                return ip
        return None



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import operator
import os
import time
import uuid

from keystoneauth1 import discover

import openstack.config
from openstack import connection
from openstack.tests import base


#: Defines the OpenStack Client Config (OCC) cloud key in your OCC config
#: file, typically in $HOME/.config/openstack/clouds.yaml. That configuration
#: will determine where the functional tests will be run and what resource
#: defaults will be used to run the functional tests.
TEST_CONFIG = openstack.config.OpenStackConfig()
TEST_CLOUD_NAME = os.getenv('OS_CLOUD', 'devstack-admin')
TEST_CLOUD_REGION = openstack.config.get_cloud_region(cloud=TEST_CLOUD_NAME)


def _get_resource_value(resource_key):
    return TEST_CONFIG.get_extra_config('functional').get(resource_key)


def _disable_keep_alive(conn):
    sess = conn.config.get_session()
    sess.keep_alive = False


class BaseFunctionalTest(base.TestCase):
    _wait_for_timeout_key = ''

    def setUp(self):
        super(BaseFunctionalTest, self).setUp()
        self.conn = connection.Connection(config=TEST_CLOUD_REGION)
        _disable_keep_alive(self.conn)

        self._demo_name = os.environ.get('OPENSTACKSDK_DEMO_CLOUD', 'devstack')
        self._demo_name_alt = os.environ.get(
            'OPENSTACKSDK_DEMO_CLOUD_ALT',
            'devstack-alt',
        )
        self._op_name = os.environ.get(
            'OPENSTACKSDK_OPERATOR_CLOUD',
            'devstack-admin',
        )

        self.config = openstack.config.OpenStackConfig()
        self._set_user_cloud()
        if self._op_name:
            self._set_operator_cloud()
        else:
            self.operator_cloud = None

        self.identity_version = self.user_cloud.config.get_api_version(
            'identity'
        )

        self.flavor = self._pick_flavor()
        self.image = self._pick_image()

        # Defines default timeout for wait_for methods used
        # in the functional tests
        self._wait_for_timeout = int(
            os.getenv(
                self._wait_for_timeout_key,
                os.getenv('OPENSTACKSDK_FUNC_TEST_TIMEOUT', 300),
            )
        )

    def _set_user_cloud(self, **kwargs):
        user_config = self.config.get_one(cloud=self._demo_name, **kwargs)
        self.user_cloud = connection.Connection(config=user_config)
        _disable_keep_alive(self.user_cloud)

        # This cloud is used by the project_cleanup test, so you can't rely on
        # it
        if self._demo_name_alt:
            user_config_alt = self.config.get_one(
                cloud=self._demo_name_alt, **kwargs
            )
            self.user_cloud_alt = connection.Connection(config=user_config_alt)
            _disable_keep_alive(self.user_cloud_alt)
        else:
            self.user_cloud_alt = None

    def _set_operator_cloud(self, **kwargs):
        operator_config = self.config.get_one(cloud=self._op_name, **kwargs)
        self.operator_cloud = connection.Connection(config=operator_config)
        _disable_keep_alive(self.operator_cloud)

    def _pick_flavor(self):
        """Pick a sensible flavor to run tests with.

        This returns None if the compute service is not present (e.g.
        ironic-only deployments).
        """
        if not self.user_cloud.has_service('compute'):
            return None

        flavors = self.user_cloud.list_flavors(get_extra=False)
        # self.add_info_on_exception('flavors', flavors)

        flavor_name = os.environ.get('OPENSTACKSDK_FLAVOR')

        if not flavor_name:
            flavor_name = _get_resource_value('flavor_name')

        if flavor_name:
            for flavor in flavors:
                if flavor.name == flavor_name:
                    return flavor

            raise self.failureException(
                "Cloud does not have flavor '%s'",
                flavor_name,
            )

        # Enable running functional tests against RAX, which requires
        # performance flavors be used for boot from volume

        for flavor in sorted(flavors, key=operator.attrgetter('ram')):
            if 'performance' in flavor.name:
                return flavor

        # Otherwise, pick the smallest flavor with a ephemeral disk configured

        for flavor in sorted(flavors, key=operator.attrgetter('ram')):
            if flavor.disk:
                return flavor

        raise self.failureException('No sensible flavor found')

    def _pick_image(self):
        """Pick a sensible image to run tests with.

        This returns None if the image service is not present.
        """
        if not self.user_cloud.has_service('image'):
            return None

        images = self.user_cloud.list_images()
        # self.add_info_on_exception('images', images)

        image_name = os.environ.get('OPENSTACKSDK_IMAGE')

        if not image_name:
            image_name = _get_resource_value('image_name')

        if image_name:
            for image in images:
                if image.name == image_name:
                    return image

            raise self.failureException(
                "Cloud does not have image '%s'",
                image_name,
            )

        for image in images:
            if image.name.startswith('cirros') and image.name.endswith('-uec'):
                return image

        for image in images:
            if (
                image.name.startswith('cirros')
                and image.disk_format == 'qcow2'
            ):
                return image

        for image in images:
            if image.name.lower().startswith('ubuntu'):
                return image
        for image in images:
            if image.name.lower().startswith('centos'):
                return image

        raise self.failureException('No sensible image found')

    def addEmptyCleanup(self, func, *args, **kwargs):
        def cleanup():
            result = func(*args, **kwargs)
            self.assertIsNone(result)

        self.addCleanup(cleanup)

    def require_service(self, service_type, min_microversion=None, **kwargs):
        """Method to check whether a service exists

        Usage::

            class TestMeter(base.BaseFunctionalTest):
                def setUp(self):
                    super(TestMeter, self).setUp()
                    self.require_service('metering')

        :returns: True if the service exists, otherwise False.
        """
        if not self.conn.has_service(service_type):
            self.skipTest(
                'Service {service_type} not found in cloud'.format(
                    service_type=service_type
                )
            )

        if not min_microversion:
            return

        data = self.conn.session.get_endpoint_data(
            service_type=service_type, **kwargs
        )

        if not (
            data.min_microversion
            and data.max_microversion
            and discover.version_between(
                data.min_microversion,
                data.max_microversion,
                min_microversion,
            )
        ):
            self.skipTest(
                f'Service {service_type} does not provide microversion '
                f'{min_microversion}'
            )

    def getUniqueString(self, prefix=None):
        """Generate unique resource name"""
        # Globally unique names can only rely on some form of uuid
        # unix_t is also used to easier determine orphans when running real
        # functional tests on a real cloud
        return (prefix if prefix else '') + "{time}-{uuid}".format(
            time=int(time.time()), uuid=uuid.uuid4().hex
        )


class KeystoneBaseFunctionalTest(BaseFunctionalTest):
    def setUp(self):
        super(KeystoneBaseFunctionalTest, self).setUp()

        use_keystone_v2 = os.environ.get('OPENSTACKSDK_USE_KEYSTONE_V2', False)
        if use_keystone_v2:
            # keystone v2 has special behavior for the admin
            # interface and some of the operations, so make a new cloud
            # object with interface set to admin.
            # We only do it for keystone tests on v2 because otherwise
            # the admin interface is not a thing that wants to actually
            # be used
            self._set_operator_cloud(interface='admin')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from keystoneauth1 import adapter

from openstack.block_storage.v3 import transfer
from openstack import resource
from openstack.tests.unit import base


FAKE_ID = "09d18b36-9e8d-4438-a4da-3f5eff5e1130"
FAKE_VOL_ID = "390de1bc-19d1-41e7-ba67-c492bb36cae5"
FAKE_VOL_NAME = "test-volume"
FAKE_TRANSFER = "7d048960-7c3f-4bf0-952f-4312fdea1dec"
FAKE_AUTH_KEY = "95bc670c0068821d"

TRANSFER = {
    "auth_key": FAKE_AUTH_KEY,
    "created_at": "2023-06-27T08:47:23.035010",
    "id": FAKE_ID,
    "name": FAKE_VOL_NAME,
    "volume_id": FAKE_VOL_ID,
}


class TestTransfer(base.TestCase):
    def setUp(self):
        super().setUp()
        self.resp = mock.Mock()
        self.resp.body = None
        self.resp.json = mock.Mock(return_value=self.resp.body)
        self.resp.headers = {}
        self.resp.status_code = 202

        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.post = mock.Mock(return_value=self.resp)
        self.sess.default_microversion = "3.55"

    def test_basic(self):
        tr = transfer.Transfer(TRANSFER)
        self.assertEqual("transfer", tr.resource_key)
        self.assertEqual("transfers", tr.resources_key)
        self.assertEqual("/volume-transfers", tr.base_path)
        self.assertTrue(tr.allow_create)
        self.assertIsNotNone(tr._max_microversion)

        self.assertDictEqual(
            {
                "limit": "limit",
                "marker": "marker",
            },
            tr._query_mapping._mapping,
        )

    @mock.patch(
        'openstack.utils.supports_microversion',
        autospec=True,
        return_value=True,
    )
    @mock.patch.object(resource.Resource, '_translate_response')
    def test_create(self, mock_mv, mock_translate):
        sot = transfer.Transfer()

        sot.create(self.sess, volume_id=FAKE_VOL_ID, name=FAKE_VOL_NAME)
        self.sess.post.assert_called_with(
            '/volume-transfers',
            json={'transfer': {}},
            microversion="3.55",
            headers={},
            params={'volume_id': FAKE_VOL_ID, 'name': FAKE_VOL_NAME},
        )

    @mock.patch(
        'openstack.utils.supports_microversion',
        autospec=True,
        return_value=False,
    )
    @mock.patch.object(resource.Resource, '_translate_response')
    def test_create_pre_v355(self, mock_mv, mock_translate):
        self.sess.default_microversion = "3.0"
        sot = transfer.Transfer()

        sot.create(self.sess, volume_id=FAKE_VOL_ID, name=FAKE_VOL_NAME)
        self.sess.post.assert_called_with(
            '/os-volume-transfer',
            json={'transfer': {}},
            microversion="3.0",
            headers={},
            params={'volume_id': FAKE_VOL_ID, 'name': FAKE_VOL_NAME},
        )

    @mock.patch(
        'openstack.utils.supports_microversion',
        autospec=True,
        return_value=True,
    )
    @mock.patch.object(resource.Resource, '_translate_response')
    def test_accept(self, mock_mv, mock_translate):
        sot = transfer.Transfer()
        sot.id = FAKE_TRANSFER

        sot.accept(self.sess, auth_key=FAKE_AUTH_KEY)
        self.sess.post.assert_called_with(
            'volume-transfers/%s/accept' % FAKE_TRANSFER,
            json={
                'accept': {
                    'auth_key': FAKE_AUTH_KEY,
                }
            },
            microversion="3.55",
        )

    @mock.patch(
        'openstack.utils.supports_microversion',
        autospec=True,
        return_value=False,
    )
    @mock.patch.object(resource.Resource, '_translate_response')
    def test_accept_pre_v355(self, mock_mv, mock_translate):
        self.sess.default_microversion = "3.0"
        sot = transfer.Transfer()
        sot.id = FAKE_TRANSFER

        sot.accept(self.sess, auth_key=FAKE_AUTH_KEY)
        self.sess.post.assert_called_with(
            'os-volume-transfer/%s/accept' % FAKE_TRANSFER,
            json={
                'accept': {
                    'auth_key': FAKE_AUTH_KEY,
                }
            },
            microversion="3.0",
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.compute.v2 import server_interface
from openstack.tests.unit import base


IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    'fixed_ips': [
        {
            'ip_address': '192.168.1.1',
            'subnet_id': 'f8a6e8f8-c2ec-497c-9f23-da9616de54ef',
        }
    ],
    'mac_addr': '2',
    'net_id': '3',
    'port_id': '4',
    'port_state': '5',
    'server_id': '6',
    'tag': '7',
}


class TestServerInterface(base.TestCase):
    def test_basic(self):
        sot = server_interface.ServerInterface()
        self.assertEqual('interfaceAttachment', sot.resource_key)
        self.assertEqual('interfaceAttachments', sot.resources_key)
        self.assertEqual('/servers/%(server_id)s/os-interface', sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = server_interface.ServerInterface(**EXAMPLE)
        self.assertEqual(EXAMPLE['fixed_ips'], sot.fixed_ips)
        self.assertEqual(EXAMPLE['mac_addr'], sot.mac_addr)
        self.assertEqual(EXAMPLE['net_id'], sot.net_id)
        self.assertEqual(EXAMPLE['port_id'], sot.port_id)
        self.assertEqual(EXAMPLE['port_state'], sot.port_state)
        self.assertEqual(EXAMPLE['server_id'], sot.server_id)
        self.assertEqual(EXAMPLE['tag'], sot.tag)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from openstack import exceptions
from openstack.tests.functional.baremetal import base


class TestBareMetalPortGroup(base.BaseBaremetalTest):
    min_microversion = '1.23'

    def setUp(self):
        super(TestBareMetalPortGroup, self).setUp()
        self.node = self.create_node()

    def test_port_group_create_get_delete(self):
        port_group = self.create_port_group()

        loaded = self.conn.baremetal.get_port_group(port_group.id)
        self.assertEqual(loaded.id, port_group.id)
        self.assertIsNotNone(loaded.node_id)

        with_fields = self.conn.baremetal.get_port_group(
            port_group.id, fields=['uuid', 'extra']
        )
        self.assertEqual(port_group.id, with_fields.id)
        self.assertIsNone(with_fields.node_id)

        self.conn.baremetal.delete_port_group(port_group, ignore_missing=False)
        self.assertRaises(
            exceptions.ResourceNotFound,
            self.conn.baremetal.get_port_group,
            port_group.id,
        )

    def test_port_list(self):
        node2 = self.create_node(name='test-node')

        pg1 = self.create_port_group(
            address='11:22:33:44:55:66', node_id=node2.id
        )
        pg2 = self.create_port_group(
            address='11:22:33:44:55:77', node_id=self.node.id
        )

        pgs = self.conn.baremetal.port_groups(address='11:22:33:44:55:77')
        self.assertEqual([p.id for p in pgs], [pg2.id])

        pgs = self.conn.baremetal.port_groups(node=node2.id)
        self.assertEqual([p.id for p in pgs], [pg1.id])

        pgs = self.conn.baremetal.port_groups(node='test-node')
        self.assertEqual([p.id for p in pgs], [pg1.id])

    def test_port_list_update_delete(self):
        self.create_port_group(
            address='11:22:33:44:55:66', extra={'foo': 'bar'}
        )
        port_group = next(
            self.conn.baremetal.port_groups(
                details=True, address='11:22:33:44:55:66'
            )
        )
        self.assertEqual(port_group.extra, {'foo': 'bar'})

        # This test checks that resources returned from listing are usable
        self.conn.baremetal.update_port_group(port_group, extra={'foo': 42})
        self.conn.baremetal.delete_port_group(port_group, ignore_missing=False)

    def test_port_group_update(self):
        port_group = self.create_port_group()
        port_group.extra = {'answer': 42}

        port_group = self.conn.baremetal.update_port_group(port_group)
        self.assertEqual({'answer': 42}, port_group.extra)

        port_group = self.conn.baremetal.get_port_group(port_group.id)
        self.assertEqual({'answer': 42}, port_group.extra)

    def test_port_group_patch(self):
        port_group = self.create_port_group()

        port_group = self.conn.baremetal.patch_port_group(
            port_group, dict(path='/extra/answer', op='add', value=42)
        )
        self.assertEqual({'answer': 42}, port_group.extra)

        port_group = self.conn.baremetal.get_port_group(port_group.id)
        self.assertEqual({'answer': 42}, port_group.extra)

    def test_port_group_negative_non_existing(self):
        uuid = "5c9dcd04-2073-49bc-9618-99ae634d8971"
        self.assertRaises(
            exceptions.ResourceNotFound,
            self.conn.baremetal.get_port_group,
            uuid,
        )
        self.assertRaises(
            exceptions.ResourceNotFound,
            self.conn.baremetal.find_port_group,
            uuid,
            ignore_missing=False,
        )
        self.assertRaises(
            exceptions.ResourceNotFound,
            self.conn.baremetal.delete_port_group,
            uuid,
            ignore_missing=False,
        )
        self.assertIsNone(self.conn.baremetal.find_port_group(uuid))
        self.assertIsNone(self.conn.baremetal.delete_port_group(uuid))

    def test_port_group_fields(self):
        self.create_node()
        self.create_port_group(address='11:22:33:44:55:66')
        result = self.conn.baremetal.port_groups(fields=['uuid', 'name'])
        for item in result:
            self.assertIsNotNone(item.id)
            self.assertIsNone(item.address)
